from django.test import TestCase

from ...models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from .. import dotlan, zkillboard, evewho, eveimageserver
from ...templatetags import evelinks


class TestEveWho(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            evewho.alliance_url(12345678),
            'https://evewho.com/alliance/12345678'
        )

    def test_corporation_url(self):
        self.assertEqual(
            evewho.corporation_url(12345678),
            'https://evewho.com/corporation/12345678'
        )

    def test_character_url(self):
        self.assertEqual(
            evewho.character_url(12345678),
            'https://evewho.com/character/12345678'
        )


class TestDotlan(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            dotlan.alliance_url('Wayne Enterprices'),
            'https://evemaps.dotlan.net/alliance/Wayne_Enterprices'
        )

    def test_corporation_url(self):
        self.assertEqual(
            dotlan.corporation_url('Wayne Technology'),
            'https://evemaps.dotlan.net/corp/Wayne_Technology'
        )
        self.assertEqual(
            dotlan.corporation_url('Crédit Agricole'),
            'https://evemaps.dotlan.net/corp/Cr%C3%A9dit_Agricole'
        )

    def test_region_url(self):
        self.assertEqual(
            dotlan.region_url('Black Rise'),
            'https://evemaps.dotlan.net/map/Black_Rise'
        )

    def test_solar_system_url(self):
        self.assertEqual(
            dotlan.solar_system_url('Jita'),
            'https://evemaps.dotlan.net/system/Jita'
        )


class TestZkillboard(TestCase):

    def test_alliance_url(self):
        self.assertEqual(
            zkillboard.alliance_url(12345678),
            'https://zkillboard.com/alliance/12345678/'
        )

    def test_corporation_url(self):
        self.assertEqual(
            zkillboard.corporation_url(12345678),
            'https://zkillboard.com/corporation/12345678/'
        )

    def test_character_url(self):
        self.assertEqual(
            zkillboard.character_url(12345678),
            'https://zkillboard.com/character/12345678/'
        )


    def test_region_url(self):
        self.assertEqual(
            zkillboard.region_url(12345678),
            'https://zkillboard.com/region/12345678/'
        )

    def test_solar_system_url(self):
        self.assertEqual(
            zkillboard.solar_system_url(12345678),
            'https://zkillboard.com/system/12345678/'
        )


class TestEveImageServer(TestCase):
    """unit test for eveimageserver"""

    def test_sizes(self):
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42),
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=32),
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=64),
            'https://images.evetech.net/characters/42/portrait?size=64'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=128),
            'https://images.evetech.net/characters/42/portrait?size=128'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=256),
            'https://images.evetech.net/characters/42/portrait?size=256'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=512),
            'https://images.evetech.net/characters/42/portrait?size=512'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, size=1024),
            'https://images.evetech.net/characters/42/portrait?size=1024'
        )
        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('corporation', 42, size=-5)

        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('corporation', 42, size=0)

        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('corporation', 42, size=31)

        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('corporation', 42, size=1025)

        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('corporation', 42, size=2048)


    def test_variant(self):
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, variant='portrait'),
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('alliance', 42, variant='logo'),
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('character', 42, variant='logo')


    def test_alliance(self):
        self.assertEqual(
            eveimageserver._eve_entity_image_url('alliance', 42),
            'https://images.evetech.net/alliances/42/logo?size=32'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('corporation', 42),
            'https://images.evetech.net/corporations/42/logo?size=32'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42),
            'https://images.evetech.net/characters/42/portrait?size=32'
        )
        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('station', 42)


    def test_tenants(self):
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, tenant='tranquility'),
            'https://images.evetech.net/characters/42/portrait?size=32&tenant=tranquility'
        )
        self.assertEqual(
            eveimageserver._eve_entity_image_url('character', 42, tenant='singularity'),
            'https://images.evetech.net/characters/42/portrait?size=32&tenant=singularity'
        )
        with self.assertRaises(ValueError):
            eveimageserver._eve_entity_image_url('character', 42, tenant='xxx')

    def test_alliance_logo_url(self):
        expected = 'https://images.evetech.net/alliances/42/logo?size=128'
        self.assertEqual(eveimageserver.alliance_logo_url(42, 128), expected)

    def test_corporation_logo_url(self):
        expected = 'https://images.evetech.net/corporations/42/logo?size=128'
        self.assertEqual(eveimageserver.corporation_logo_url(42, 128), expected)

    def test_character_portrait_url(self):
        expected = 'https://images.evetech.net/characters/42/portrait?size=128'
        self.assertEqual(
            eveimageserver.character_portrait_url(42, 128), expected
        )

    def test_type_icon_url(self):
        expected = 'https://images.evetech.net/types/42/icon?size=128'
        self.assertEqual(eveimageserver.type_icon_url(42, 128), expected)

    def test_type_render_url(self):
        expected = 'https://images.evetech.net/types/42/render?size=128'
        self.assertEqual(eveimageserver.type_render_url(42, 128), expected)



# Generated by Django 4.2 on 2024-09-09 03:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0005_alter_timer_planet_moon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='objective',
            field=models.CharField(choices=[('Friendly', 'Friendly'), ('Hostile', 'Hostile'), ('Neutral', 'Neutral')], default='Neutral', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='structure',
            field=models.CharField(choices=[('POCO', 'POCO'), ('Orbital Skyhook', 'Orbital Skyhook'), ('I-HUB', 'I-HUB'), ('TCU', 'TCU'), ('POS[S]', 'POS [S]'), ('POS[M]', 'POS [M]'), ('POS[L]', 'POS [L]'), ('Astrahus', 'Astrahus'), ('Fortizar', 'Fortizar'), ('Keepstar', 'Keepstar'), ('Raitaru', 'Raitaru'), ('Azbel', 'Azbel'), ('Sotiyo', 'Sotiyo'), ('Athanor', 'Athanor'), ('Tatara', 'Tatara'), ('Pharolux Cyno Beacon', 'Pharolux Cyno Beacon'), ('Tenebrex Cyno Jammer', 'Tenebrex Cyno Jammer'), ('Ansiblex Jump Gate', 'Ansiblex Jump Gate'), ('Moon Mining Cycle', 'Moon Mining Cycle'), ('Metenox Moon Drill', 'Metenox Moon Drill'), ('Other', 'Other')], default='Other', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='timer_type',
            field=models.CharField(choices=[('UNSPECIFIED', 'Not Specified'), ('SHIELD', 'Shield'), ('ARMOR', 'Armor'), ('HULL', 'Hull'), ('FINAL', 'Final'), ('ANCHORING', 'Anchoring'), ('UNANCHORING', 'Unanchoring'), ('ABANDONED', 'Abandoned')], default='UNSPECIFIED', max_length=254),
        ),
    ]



from allianceauth.menu.hooks import MenuItemHook
from . import urls
from django.utils.translation import gettext_lazy as _
from allianceauth import hooks
from allianceauth.services.hooks import UrlHook


@hooks.register('menu_item_hook')
def register_menu():
    return MenuItemHook(_('Fleet Activity Tracking'), 'fa-solid fa-users', 'fatlink:view',
                        navactive=['fatlink:'])


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'fatlink', r'^fat/')



# Generated by Django 3.2.10 on 2022-01-05 18:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0015_factions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='evecharacter',
            name='character_name',
            field=models.CharField(db_index=True, max_length=254),
        ),
    ]



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect
from django.utils.translation import gettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm

from .manager import SmfManager
from .models import SmfUser
from .tasks import SmfTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'smf.access_smf'


@login_required
@permission_required(ACCESS_PERM)
def activate_smf(request):
    logger.debug(f"activate_smf called by user {request.user}")
    # Valid now we get the main characters
    main_character = request.user.profile.main_character

    logger.debug(
        f"Adding SMF user for user {request.user} with main character {main_character}"
    )

    result = SmfManager.add_user(
        SmfTasks.get_username(request.user),
        request.user.email,
        ['Member'],
        main_character,
    )

    # if empty we failed
    if result[0] != "":
        SmfUser.objects.update_or_create(
            user=request.user, defaults={'username': result[0]}
        )

        logger.debug(
            f"Updated authserviceinfo for user {request.user} "
            f"with SMF credentials. Updating groups."
        )

        SmfTasks.update_groups.delay(request.user.pk)

        logger.info(f"Successfully activated SMF for user {request.user}")

        messages.success(request, _('Activated SMF account.'))
        credentials = {
            'username': result[0],
            'password': result[1],
        }

        return render(
            request,
            'services/service_credentials.html',
            context={'credentials': credentials, 'service': 'SMF'},
        )

    logger.error(f"Unsuccessful attempt to activate SMF for user {request.user}")
    messages.error(request, _('An error occurred while processing your SMF account.'))

    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_smf(request):
    logger.debug("deactivate_smf called by user %s" % request.user)
    result = SmfTasks.delete_user(request.user)
    # false we failed
    if result:
        logger.info("Successfully deactivated SMF for user %s" % request.user)
        messages.success(request, _('Deactivated SMF account.'))
    else:
        logger.error("Unsuccessful attempt to activate SMF for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your SMF account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_smf_password(request):
    logger.debug("reset_smf_password called by user %s" % request.user)
    character = request.user.profile.main_character
    if SmfTasks.has_account(request.user) and character is not None:
        result = SmfManager.update_user_password(request.user.smf.username, character.character_id)
        # false we failed
        if result != "":
            logger.info("Successfully reset SMF password for user %s" % request.user)
            messages.success(request, _('Reset SMF password.'))
            credentials = {
                'username': request.user.smf.username,
                'password': result,
            }
            return render(request, 'services/service_credentials.html', context={'credentials': credentials, 'service': 'SMF'})
    logger.error("Unsuccessful attempt to reset SMF password for user %s" % request.user)
    messages.error(request, _('An error occurred while processing your SMF account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_smf_password(request):
    logger.debug("set_smf_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        character = request.user.profile.main_character
        if form.is_valid() and SmfTasks.has_account(request.user) and character is not None:
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = SmfManager.update_user_password(request.user.smf.username, character.character_id, password=password)
            if result != "":
                logger.info("Successfully set SMF password for user %s" % request.user)
                messages.success(request, _('Set SMF password.'))
            else:
                logger.error("Failed to install custom SMF password for user %s" % request.user)
                messages.error(request, _('An error occurred while processing your SMF account.'))
            return redirect("services:services")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'SMF'}
    return render(request, 'services/service_password.html', context=context)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class XenforoServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.xenforo'
    label = 'xenforo'
    verbose_name = _('Xenforo Service')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DiscordServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.discord'
    label = 'discord'
    verbose_name = _('Discord Service')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class SmfServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.smf'
    label = 'smf'
    verbose_name = _('SMF Service')



from allianceauth.menu.hooks import MenuItemHook
from allianceauth.optimer.views import dashboard_ops
from allianceauth.services.hooks import UrlHook
from django.utils.translation import gettext_lazy as _
from allianceauth import hooks
from . import urls


class OpTimerboardMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(
            self, _('Fleet Operations'),
            'fa-solid fa-exclamation',
            'optimer:view',
            navactive=['optimer:']
        )

    def render(self, request):
        if request.user.has_perm('auth.optimer_view'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return OpTimerboardMenu()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'optimer', r'^optimer/')


class NextOpsHook(hooks.DashboardItemHook):
    def __init__(self): #TODO add the view perms so if they cant see it is not rendered
        hooks.DashboardItemHook.__init__(
            self,
            dashboard_ops,
            6
        )


@hooks.register('dashboard_hook')
def register_groups_hook():
    return NextOpsHook()



"""
Custom static files storage for Alliance Auth.

This module defines a custom static files storage class for
Alliance Auth, named `AaManifestStaticFilesStorage`.

Using `ManifestStaticFilesStorage` will give us a hashed name for
our static files, which is useful for cache busting.

This storage class extends Django's `ManifestStaticFilesStorage` to ignore missing files,
which the original class does not handle, and log them in debug mode.
It is useful for handling cases where static files may not exist, such as when a
CSS file references a background image that is not present in the static files directory.

With debug mode enabled, it will print a message for each missing file when running `collectstatic`,
which can help identify issues with static file references during development.
"""

from django.conf import settings
from django.contrib.staticfiles.storage import ManifestStaticFilesStorage


class AaManifestStaticFilesStorage(ManifestStaticFilesStorage):
    """
    Custom static files storage that ignores missing files.
    """

    @classmethod
    def _cleanup_name(cls, name: str) -> str:
        """
        Clean up the name by removing quotes.
        This method is used to ensure that the name does not contain any quotes,
        which can cause issues with file paths.

        :param name: The name of the static file.
        :type name: str
        :return: The cleaned-up name without quotes.
        :rtype: str
        """

        # Remove quotes from the name
        return name.replace('"', "").replace("'", "")

    def __init__(self, *args, **kwargs):
        """
        Initialize the static files storage, ignoring missing files.

        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        """

        self.missing_files = []

        super().__init__(*args, **kwargs)

    def hashed_name(self, name, content=None, filename=None):
        """
        Generate a hashed name for the given static file, ignoring missing files.

        Ignore missing files, e.g. non-existent background image referenced from css.
        Returns the original filename if the referenced file doesn't exist.

        :param name: The name of the static file to hash.
        :type name: str
        :param content: The content of the static file, if available.
        :type content: bytes | None
        :param filename: The original filename of the static file, if available.
        :type filename: str | None
        :return: The hashed name of the static file, or the original name if the file is missing.
        :rtype: str
        """

        try:
            clean_name = self._cleanup_name(name)

            return super().hashed_name(clean_name, content, filename)
        except ValueError as e:
            if settings.DEBUG:
                # In debug mode, we log the missing file message
                message = e.args[0].split(" with ")[0]
                self.missing_files.append(message)
                # print(f'\x1b[0;30;41m{message}\x1b[0m')

            return name

    def post_process(self, *args, **kwargs):
        """
        Post-process the static files, printing any missing files in debug mode.

        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """

        yield from super().post_process(*args, **kwargs)

        if settings.DEBUG:
            # In debug mode, print the missing files
            for message in sorted(set(self.missing_files)):
                print(f"\x1b[0;30;41m{message}\x1b[0m")



from random import random
from django.db import models
from django.utils.translation import gettext_lazy as _
from solo.models import SingletonModel


def random_default() -> float:
    return random()


class CronOffset(SingletonModel):

    minute = models.FloatField(_("Minute Offset"), default=random_default)
    hour = models.FloatField(_("Hour Offset"), default=random_default)
    day_of_month = models.FloatField(_("Day of Month Offset"), default=random_default)
    month_of_year = models.FloatField(_("Month of Year Offset"), default=random_default)
    day_of_week = models.FloatField(_("Day of Week Offset"), default=random_default)

    def __str__(self) -> str:
        return "Cron Offsets"

    class Meta:
        verbose_name = "Cron Offsets"



from django import forms
from django.utils.translation import gettext_lazy as _

from allianceauth.optimer.form_widgets import DataListWidget


class OpForm(forms.Form):
    """
    Create/Edit Fleet Operation Form
    """

    doctrine = forms.CharField(max_length=254, required=True, label=_('Doctrine'))
    system = forms.CharField(max_length=254, required=True, label=_("System"))
    start = forms.DateTimeField(required=True, label=_("Start Time"))
    operation_name = forms.CharField(max_length=254, required=True, label=_("Operation Name"))
    type = forms.CharField(required=False, label=_("Operation Type"))
    fc = forms.CharField(max_length=254, required=True, label=_("Fleet Commander"))
    duration = forms.CharField(max_length=25, required=True, label=_("Duration"))
    description = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 10, "cols": 20, "input_type": "textarea"}),
        required=False,
        label=_("Additional Info"),
        help_text=_("(Optional) Describe the operation with a couple of short words."),
    )

    def __init__(self, *args, **kwargs):
        _data_list = kwargs.pop('data_list', None)

        super().__init__(*args, **kwargs)

        # Add the DataListWidget to our type field
        self.fields['type'].widget = DataListWidget(
            data_list=_data_list, name='data-list'
        )



from django.urls import path

from . import views
app_name = 'fleetactivitytracking'

urlpatterns = [
    # FleetActivityTracking (FAT)
    path('', views.fatlink_view, name='view'),
    path('statistics/', views.fatlink_statistics_view, name='statistics'),
    path('statistics/corp/<int:corpid>/', views.fatlink_statistics_corp_view,
        name='statistics_corp'),
    path('statistics/corp/<int:corpid>/<int:year>/<int:month>/',
        views.fatlink_statistics_corp_view,
        name='statistics_corp_month'),
    path('statistics/<int:year>/<int:month>/', views.fatlink_statistics_view,
        name='statistics_month'),
    path('user/statistics/', views.fatlink_personal_statistics_view,
        name='personal_statistics'),
    path('user/statistics/<int:year>/', views.fatlink_personal_statistics_view,
        name='personal_statistics_year'),
    path('user/statistics/<int:year>/<int:month>/',
        views.fatlink_monthly_personal_statistics_view,
        name='personal_statistics_month'),
    path('user/<int:char_id>/statistics/<int:year>/<int:month>/',
        views.fatlink_monthly_personal_statistics_view,
        name='user_statistics_month'),
    path('create/', views.create_fatlink_view, name='create'),
    path('modify/<str:fat_hash>/', views.modify_fatlink_view, name='modify'),
    path('link/<str:fat_hash>/', views.click_fatlink_view, name='click'),
]



from unittest import TestCase

from allianceauth.services.hooks import UrlHook
from allianceauth.groupmanagement import urls


class TestUrlHook(TestCase):
    def test_can_create_simple_hook(self):
        # when
        obj = UrlHook(urls, "groupmanagement", r"^groupmanagement/")
        # then
        self.assertEqual(obj.include_pattern.app_name, "groupmanagement")
        self.assertFalse(obj.excluded_views)

    def test_can_create_hook_with_excluded_views(self):
        # when
        obj = UrlHook(
            urls,
            "groupmanagement",
            r"^groupmanagement/",
            ["groupmanagement.views.group_management"],
        )
        # then
        self.assertEqual(obj.include_pattern.app_name, "groupmanagement")
        self.assertIn("groupmanagement.views.group_management", obj.excluded_views)

    def test_should_raise_error_when_called_with_invalid_excluded_views(self):
        # when/then
        with self.assertRaises(TypeError):
            UrlHook(urls, "groupmanagement", r"^groupmanagement/", 99)



"""
Crontab App Config
"""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CrontabConfig(AppConfig):
    """
    Crontab App Config
    """

    name = "allianceauth.crontab"
    label = "crontab"
    verbose_name = _("Crontab")



import logging

from django.conf import settings
from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import SmfTasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class SmfService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'smf'
        self.urlpatterns = urlpatterns
        self.service_url = settings.SMF_URL
        self.access_perm = 'smf.access_smf'
        self.name_format = '{character_name}'

    @property
    def title(self):
        return 'SMF Forums'

    def delete_user(self, user, notify_user=False):
        logger.debug(f'Deleting user {user} {self.name} account')
        return SmfTasks.delete_user(user, notify_user=notify_user)

    def validate_user(self, user):
        logger.debug(f'Validating user {user} {self.name} account')
        if SmfTasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user)

    def update_groups(self, user):
        logger.debug(f'Updating {self.name} groups for {user}')
        if SmfTasks.has_account(user):
            SmfTasks.update_groups.delay(user.pk)

    def sync_nickname(self, user):
        logger.debug(f"Updating {self.name} displayed name for {user}")

        if SmfTasks.has_account(user):
            SmfTasks.update_display_name.apply_async(args=[user.pk], countdown=5) # cooldown on this task to ensure DB clean when syncing

    def update_all_groups(self):
        logger.debug('Update all %s groups called' % self.name)
        SmfTasks.update_all_groups.delay()

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        urls = self.Urls()
        urls.auth_activate = 'smf:activate'
        urls.auth_deactivate = 'smf:deactivate'
        urls.auth_reset_password = 'smf:reset_password'
        urls.auth_set_password = 'smf:set_password'
        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': self.service_url,
            'username': request.user.smf.username if SmfTasks.has_account(request.user) else ''
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return SmfService()



# Generated by Django 1.11.5 on 2017-10-07 06:43

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('services', '0001_squashed_0003_delete_groupcache'),
        ('authentication', '0015_user_profiles'),
    ]

    operations = [
        migrations.CreateModel(
            name='NameFormatConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('service_name', models.CharField(max_length=100)),
                ('default_to_username', models.BooleanField(default=True, help_text='If a user has no main_character, default to using their Auth username instead.')),
                ('format', models.CharField(help_text='For information on constructing name formats, please see the <a href="https://allianceauth.readthedocs.io/en/latest/features/nameformats">name format documentation</a>', max_length=100)),
                ('states', models.ManyToManyField(help_text='States to apply this format to. You should only have one formatter for each state for each service.', to='authentication.State')),
            ],
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class Teamspeak3ServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.teamspeak3'
    label = 'teamspeak3'
    verbose_name = _('TeamSpeak 3 Service')

    def ready(self):
        from . import signals



"""Menu item hooks."""

from typing import List, Optional

from django.template.loader import render_to_string

from allianceauth.menu.constants import DEFAULT_MENU_ITEM_ORDER


class MenuItemHook:
    """Auth Hook for generating side menu items.

    Args:
        - text: The text shown as menu item, e.g. usually the name of the app.
        - classes: The classes that should be applied to the menu item icon
        - url_name: The name of the Django URL to use
        - order: An integer which specifies the order of the menu item,
            lowest to highest. Community apps are free to use any order above `1000`.
            Numbers below are served for Auth.
        - A list of views or namespaces the link should be highlighted on.
            See 3rd party package django-navhelper for usage.
                Defaults to the supplied `url_name`.


    Optional:
        - count is an integer shown next to the menu item as badge when is is not `None`.
            Apps need to set the count in their child class, e.g. in `render()` method

    """

    def __init__(
        self,
        text: str,
        classes: str,
        url_name: str,
        order: Optional[int] = None,
        navactive: Optional[List[str]] = None,
    ):
        self.text = text
        self.classes = classes
        self.url_name = url_name
        self.template = "public/menuitem.html"
        self.order = order if order is not None else DEFAULT_MENU_ITEM_ORDER
        self.count = None

        navactive = navactive or []
        navactive.append(url_name)
        self.navactive = navactive

    def __str__(self) -> str:
        return self.text

    def __repr__(self) -> str:
        return f'{self.__class__.__name__}(text="{self.text}")'

    def render(self, request) -> str:
        """Render this menu item and return resulting HTML."""
        return render_to_string(self.template, {"item": self}, request=request)



# Generated by Django 1.11.6 on 2017-10-09 09:19

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0006_service_permissions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='mumbleuser',
            name='id',
        ),
        migrations.AlterField(
            model_name='mumbleuser',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='mumble', serialize=False, to=settings.AUTH_USER_MODEL),
        ),
    ]



from typing import Literal
from django.db import models
from django.utils.translation import gettext_lazy as _
from solo.models import SingletonModel
from uuid import uuid4


class AnalyticsIdentifier(SingletonModel):

    identifier = models.UUIDField(default=uuid4, editable=False)

    def __str__(self) -> Literal['Analytics Identifier']:
        return "Analytics Identifier"

    class Meta:
        verbose_name = "Analytics Identifier"


class AnalyticsTokens(models.Model):

    class Analytics_Type(models.TextChoices):
        GA_U = 'GA-U', _('Google Analytics Universal')
        GA_V4 = 'GA-V4', _('Google Analytics V4')

    name = models.CharField(max_length=254)
    type = models.CharField(max_length=254, choices=Analytics_Type.choices)
    token = models.CharField(max_length=254, blank=False)
    secret = models.CharField(max_length=254, blank=True)
    send_stats = models.BooleanField(default=False)



"""
Form Widgets
"""

from django import forms
from django.utils.safestring import mark_safe


class DataListWidget(forms.TextInput):
    """
    DataListWidget

    Draws an HTML5 datalist form field
    """

    def __init__(self, data_list, name, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._name = name
        self._list = data_list
        self.attrs.update({"list": "list__%s" % self._name})

    def render(self, name, value, attrs=None, renderer=None):
        """
        Render the DataList
        :param name:
        :type name:
        :param value:
        :type value:
        :param attrs:
        :type attrs:
        :param renderer:
        :type renderer:
        :return:
        :rtype:
        """

        text_html = super().render(name, value, attrs=attrs)
        data_list = '<datalist id="list__%s">' % self._name

        for item in self._list:
            data_list += '<option value="%s">' % item

        data_list += "</datalist>"

        return mark_safe(text_html + data_list)



from django.test import TestCase

from allianceauth.menu.constants import DEFAULT_FOLDER_ICON_CLASSES
from allianceauth.menu.forms import FolderMenuItemAdminForm


class TestFolderMenuItemAdminForm(TestCase):
    def test_should_set_default_icon_classes(self):
        # given
        form_data = {"text": "Alpha", "order": 1}
        form = FolderMenuItemAdminForm(data=form_data)

        # when
        obj = form.save(commit=False)

        # then
        self.assertEqual(obj.classes, DEFAULT_FOLDER_ICON_CLASSES)

    def test_should_use_icon_classes_from_input(self):
        # given
        form_data = {"text": "Alpha", "order": 1, "classes": "dummy"}
        form = FolderMenuItemAdminForm(data=form_data)

        # when
        obj = form.save(commit=False)

        # then
        self.assertEqual(obj.classes, "dummy")



"""Filters for the menu app."""

from django.contrib import admin
from django.utils.translation import gettext_noop as _

from allianceauth.menu.constants import MenuItemType


class MenuItemTypeListFilter(admin.SimpleListFilter):
    """Allow filtering admin changelist by menu item type."""

    title = _("type")
    parameter_name = "type"

    def lookups(self, request, model_admin):
        return [(obj.value, obj.label.title()) for obj in MenuItemType]

    def queryset(self, request, queryset):
        if value := self.value():
            return queryset.annotate_item_type_2().filter(
                item_type_2=MenuItemType(value).value
            )

        return None



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class OptimerConfig(AppConfig):
    name = 'allianceauth.optimer'
    label = 'optimer'
    verbose_name = _('Fleet Operations')



from django.urls import path
from . import views

app_name = 'notifications'
# Notifications
urlpatterns = [
    path('remove_notifications/<int:notif_id>/', views.remove_notification, name='remove'),
    path('notifications/mark_all_read/', views.mark_all_read, name='mark_all_read'),
    path('notifications/delete_all_read/', views.delete_all_read, name='delete_all_read'),
    path('notifications/', views.notification_list, name='list'),
    path('notifications/<int:notif_id>/', views.notification_view, name='view'),
    path(
        'user_notifications_count/<int:user_pk>/',
        views.user_notifications_count,
        name='user_notifications_count'
    ),
]



# Generated by Django 2.2.12 on 2020-05-26 02:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0011_ids_to_integers'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='eveallianceinfo',
            index=models.Index(fields=['executor_corp_id'], name='eveonline_e_executo_7f3280_idx'),
        ),
        migrations.AddIndex(
            model_name='evecharacter',
            index=models.Index(fields=['corporation_id'], name='eveonline_e_corpora_cb4cd9_idx'),
        ),
        migrations.AddIndex(
            model_name='evecharacter',
            index=models.Index(fields=['alliance_id'], name='eveonline_e_allianc_39ee2a_idx'),
        ),
        migrations.AddIndex(
            model_name='evecharacter',
            index=models.Index(fields=['corporation_name'], name='eveonline_e_corpora_893c60_idx'),
        ),
        migrations.AddIndex(
            model_name='evecharacter',
            index=models.Index(fields=['alliance_name'], name='eveonline_e_allianc_63fd98_idx'),
        ),
    ]



from http import HTTPStatus

from django.test import TestCase
from django.urls import reverse

from allianceauth.menu.constants import MenuItemType
from allianceauth.menu.forms import (
    AppMenuItemAdminForm,
    FolderMenuItemAdminForm,
    LinkMenuItemAdminForm,
)
from allianceauth.menu.models import MenuItem
from allianceauth.menu.tests.factories import (
    create_app_menu_item,
    create_folder_menu_item,
    create_link_menu_item,
    create_user,
)
from allianceauth.menu.tests.utils import extract_html


def extract_menu_item_texts(response):
    """Extract labels of menu items shown in change list."""
    soup = extract_html(response)
    items = soup.find_all("th", {"class": "field-_text"})
    labels = {elem.text for elem in items}
    return labels


class TestAdminSite(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = create_user(is_superuser=True, is_staff=True)
        cls.changelist_url = reverse("admin:menu_menuitem_changelist")
        cls.add_url = reverse("admin:menu_menuitem_add")

    def change_url(self, id_):
        return reverse("admin:menu_menuitem_change", args=[id_])

    def test_changelist_should_show_all_types(self):
        # given
        self.client.force_login(self.user)
        create_app_menu_item(text="app")
        create_folder_menu_item(text="folder")
        create_link_menu_item(text="link")

        # when
        response = self.client.get(self.changelist_url)

        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        labels = extract_menu_item_texts(response)
        self.assertSetEqual(labels, {"app", "[folder]", "link"})

    def test_should_create_new_link_item(self):
        # given
        self.client.force_login(self.user)

        # when
        response = self.client.post(
            self.add_url,
            {"text": "alpha", "url": "http://www.example.com", "order": 99},
        )
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        self.assertEqual(MenuItem.objects.count(), 1)
        obj = MenuItem.objects.first()
        self.assertEqual(obj.text, "alpha")
        self.assertEqual(obj.item_type, MenuItemType.LINK)

    def test_should_create_new_folder_item(self):
        # given
        self.client.force_login(self.user)

        # when
        response = self.client.post(
            self.add_url + "?type=folder", {"text": "alpha", "order": 99}
        )
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        self.assertEqual(MenuItem.objects.count(), 1)
        obj = MenuItem.objects.first()
        self.assertEqual(obj.text, "alpha")
        self.assertEqual(obj.item_type, MenuItemType.FOLDER)

    def test_should_change_app_item(self):
        # given
        self.client.force_login(self.user)
        item = create_app_menu_item(text="alpha", order=1)
        form_data = AppMenuItemAdminForm(instance=item).initial
        form_data["order"] = 99
        form_data["parent"] = ""

        # when
        response = self.client.post(self.change_url(item.id), form_data)

        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        self.assertEqual(MenuItem.objects.count(), 1)
        obj = MenuItem.objects.first()
        self.assertEqual(obj.order, 99)
        self.assertEqual(obj.item_type, MenuItemType.APP)

    def test_should_change_link_item(self):
        # given
        self.client.force_login(self.user)
        item = create_link_menu_item(text="alpha")
        form_data = LinkMenuItemAdminForm(instance=item).initial
        form_data["text"] = "bravo"
        form_data["parent"] = ""

        # when
        response = self.client.post(self.change_url(item.id), form_data)

        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        self.assertEqual(MenuItem.objects.count(), 1)
        obj = MenuItem.objects.first()
        self.assertEqual(obj.text, "bravo")
        self.assertEqual(obj.item_type, MenuItemType.LINK)

    def test_should_change_folder_item(self):
        # given
        self.client.force_login(self.user)
        item = create_folder_menu_item(text="alpha")
        form_data = FolderMenuItemAdminForm(instance=item).initial
        form_data["text"] = "bravo"

        # when
        response = self.client.post(self.change_url(item.id), form_data)

        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        self.assertEqual(MenuItem.objects.count(), 1)
        obj = MenuItem.objects.first()
        self.assertEqual(obj.text, "bravo")
        self.assertEqual(obj.item_type, MenuItemType.FOLDER)

    def test_should_move_item_into_folder(self):
        # given
        self.client.force_login(self.user)
        link = create_link_menu_item(text="alpha")
        folder = create_folder_menu_item(text="folder")
        form_data = LinkMenuItemAdminForm(instance=link).initial
        form_data["parent"] = folder.id

        # when
        response = self.client.post(self.change_url(link.id), form_data)

        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.changelist_url)
        link.refresh_from_db()
        self.assertEqual(link.parent, folder)

    def test_should_filter_items_by_type(self):
        # given
        self.client.force_login(self.user)
        create_app_menu_item(text="app")
        create_folder_menu_item(text="folder")
        create_link_menu_item(text="link")

        # when
        cases = [("link", "link"), ("app", "app"), ("folder", "[folder]")]
        for filter_name, expected_label in cases:
            with self.subTest(filter_name=filter_name):
                response = self.client.get(self.changelist_url + f"?type={filter_name}")

                # then
                self.assertEqual(response.status_code, HTTPStatus.OK)
                labels = extract_menu_item_texts(response)
                self.assertSetEqual(labels, {expected_label})



from allianceauth.utils.testing import NoSocketsTestCase

from ..helpers import RolesSet
from .factories import create_matched_role, create_role


class TestRolesSet(NoSocketsTestCase):
    def test_can_create_simple(self):
        # given
        roles_raw = [create_role()]
        # when
        roles = RolesSet(roles_raw)
        # then
        self.assertListEqual(list(roles), roles_raw)

    def test_can_create_empty(self):
        # when
        roles = RolesSet([])
        # then
        self.assertListEqual(list(roles), [])

    def test_raises_exception_if_roles_raw_of_wrong_type(self):
        with self.assertRaises(TypeError):
            RolesSet({"id": 1})

    def test_raises_exception_if_list_contains_non_dict(self):
        # given
        roles_raw = [create_role(), "not_valid"]
        # when/then
        with self.assertRaises(TypeError):
            RolesSet(roles_raw)

    def test_roles_are_equal(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles_a = RolesSet([role_a, role_b])
        roles_b = RolesSet([role_a, role_b])
        # when/then
        self.assertEqual(roles_a, roles_b)

    def test_roles_are_not_equal(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles_a = RolesSet([role_a, role_b])
        roles_b = RolesSet([role_a])
        # when/then
        self.assertNotEqual(roles_a, roles_b)

    def test_different_objects_are_not_equal(self):
        roles_a = RolesSet([])
        self.assertFalse(roles_a == "invalid")

    def test_len(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles = RolesSet([role_a, role_b])
        # when/then
        self.assertEqual(len(roles), 2)

    def test_contains(self):
        # given
        role_a = create_role(id=1)
        roles = RolesSet([role_a])
        # when/then
        self.assertTrue(1 in roles)
        self.assertFalse(99 in roles)

    def test_objects_are_hashable(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_c = create_role()
        roles_a = RolesSet([role_a, role_b])
        roles_b = RolesSet([role_b, role_a])
        roles_c = RolesSet([role_a, role_b, role_c])
        # when/then
        self.assertIsNotNone(hash(roles_a))
        self.assertEqual(hash(roles_a), hash(roles_b))
        self.assertNotEqual(hash(roles_a), hash(roles_c))

    def test_create_from_matched_roles(self):
        role_a = create_role()
        role_b = create_role()
        matched_roles = [
            create_matched_role(role_a, True),
            create_matched_role(role_b, False),
        ]
        # when
        roles = RolesSet.create_from_matched_roles(matched_roles)
        # then
        self.assertEqual(roles, RolesSet([role_a, role_b]))

    def test_return_role_ids_default(self):
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        roles = RolesSet([role_a, role_b])
        # when/then
        self.assertSetEqual(roles.ids(), {1, 2})

    def test_return_role_ids_empty(self):
        # given
        roles = RolesSet([])
        # when/then
        self.assertSetEqual(roles.ids(), set())


class TestRolesSetSubset(NoSocketsTestCase):
    def test_ids_only(self):
        # given
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        role_c = create_role(id=3)
        roles_all = RolesSet([role_a, role_b, role_c])
        # when
        roles_subset = roles_all.subset({1, 3})
        # then
        self.assertEqual(roles_subset, RolesSet([role_a, role_c]))

    def test_ids_as_string_work_too(self):
        # given
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        role_c = create_role(id=3)
        roles_all = RolesSet([role_a, role_b, role_c])
        # when
        roles_subset = roles_all.subset({"1", "3"})
        # then
        self.assertEqual(roles_subset, RolesSet([role_a, role_c]))

    def test_managed_only(self):
        # given
        role_a = create_role(id=1)
        role_m = create_role(id=13, managed=True)
        roles_all = RolesSet([role_a, role_m])
        # when
        roles_subset = roles_all.subset(managed_only=True)
        # then
        self.assertEqual(roles_subset, RolesSet([role_m]))

    def test_ids_and_managed_only(self):
        # given
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        role_m = create_role(id=13, managed=True)
        roles_all = RolesSet([role_a, role_b, role_m])
        # when
        roles_subset = roles_all.subset({1, 13}, managed_only=True)
        # then
        self.assertEqual(roles_subset, RolesSet([role_m]))

    def test_ids_are_empty(self):
        # given
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        roles_all = RolesSet([role_a, role_b])
        roles_subset = roles_all.subset([])
        # then
        self.assertEqual(roles_subset, RolesSet([]))

    def test_no_parameters(self):
        # given
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        roles_all = RolesSet([role_a, role_b])
        roles_subset = roles_all.subset()
        # then
        self.assertEqual(roles_subset, roles_all)

    def test_should_return_role_names_only(self):
        # given
        role_a = create_role(name="alpha")
        role_b = create_role(name="bravo")
        role_c1 = create_role(name="charlie")
        role_c2 = create_role(name="Charlie")
        roles_all = RolesSet([role_a, role_b, role_c1, role_c2])
        # when
        roles_subset = roles_all.subset(role_names={"bravo", "charlie"})
        # then
        self.assertSetEqual(roles_subset, RolesSet([role_b, role_c1, role_c2]))


class TestRolesSetHasRoles(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        role_a = create_role(id=1)
        role_b = create_role(id=2)
        role_c = create_role(id=3)
        cls.all_roles = RolesSet([role_a, role_b, role_c])

    def test_true_if_all_roles_exit(self):
        self.assertTrue(self.all_roles.has_roles([1, 2]))

    def test_true_if_all_roles_exit_str(self):
        self.assertTrue(self.all_roles.has_roles(["1", "2"]))

    def test_false_if_role_does_not_exit(self):
        self.assertFalse(self.all_roles.has_roles([99]))

    def test_false_if_one_role_does_not_exit(self):
        self.assertFalse(self.all_roles.has_roles([1, 99]))

    def test_true_for_empty_roles(self):
        self.assertTrue(self.all_roles.has_roles([]))


class TestRolesSetGetMatchingRolesByName(NoSocketsTestCase):
    def test_return_role_if_matches(self):
        # given
        role_a = create_role(name="alpha")
        role_b = create_role(name="bravo")
        roles = RolesSet([role_a, role_b])
        # when
        result = roles.role_by_name("alpha")
        # then
        self.assertEqual(result, role_a)

    def test_return_role_if_matches_and_limit_max_length(self):
        # given
        role_name = "x" * 120
        role = create_role(name="x" * 100)
        roles = RolesSet([role])
        # when
        result = roles.role_by_name(role_name)
        # then
        self.assertEqual(result, role)

    def test_return_empty_if_not_matches(self):
        # given
        role_a = create_role(name="alpha")
        role_b = create_role(name="bravo")
        roles = RolesSet([role_a, role_b])
        # when
        result = roles.role_by_name("unknown")
        # then
        self.assertIsNone(result)


class TestRolesSetUnion(NoSocketsTestCase):
    def test_distinct_sets(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles_1 = RolesSet([role_a])
        roles_2 = RolesSet([role_b])
        # when
        result = roles_1.union(roles_2)
        # then
        self.assertEqual(result, RolesSet([role_a, role_b]))

    def test_overlapping_sets(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_c = create_role()
        roles_1 = RolesSet([role_a, role_b])
        roles_2 = RolesSet([role_b, role_c])
        # when
        result = roles_1.union(roles_2)
        self.assertEqual(result, RolesSet([role_a, role_b, role_c]))

    def test_identical_sets(self):
        role_a = create_role()
        role_b = create_role()
        roles_1 = RolesSet([role_a, role_b])
        roles_2 = RolesSet([role_a, role_b])
        # when
        result = roles_1.union(roles_2)
        self.assertEqual(result, RolesSet([role_a, role_b]))


class TestRolesSetDifference(NoSocketsTestCase):
    def test_distinct_sets(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_c = create_role()
        role_d = create_role()
        roles_1 = RolesSet([role_a, role_b])
        roles_2 = RolesSet([role_c, role_d])
        # when
        result = roles_1.difference(roles_2)
        # then
        self.assertEqual(result, RolesSet([role_a, role_b]))

    def test_overlapping_sets(self):
        # given
        role_a = create_role()
        role_b = create_role()
        role_c = create_role()
        roles_1 = RolesSet([role_a, role_b])
        roles_2 = RolesSet([role_b, role_c])
        # when
        result = roles_1.difference(roles_2)
        # then
        self.assertEqual(result, RolesSet([role_a]))

    def test_identical_sets(self):
        # given
        role_a = create_role()
        role_b = create_role()
        roles_1 = RolesSet([role_a, role_b])
        roles_2 = RolesSet([role_a, role_b])
        # when
        result = roles_1.difference(roles_2)
        # then
        self.assertEqual(result, RolesSet([]))



# Generated by Django 3.1.4 on 2020-12-30 08:53

from django.db import migrations
from django.core.exceptions import ObjectDoesNotExist


def add_aa_team_token(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    AnalyticsPath = apps.get_model('analytics', 'AnalyticsPath')
    token = Tokens()
    try:
        ua_token = Tokens.objects.get(token="UA-186249766-2")
        original_send_page_views = ua_token.send_page_views
        original_send_celery_tasks = ua_token.send_celery_tasks
        original_send_stats = ua_token.send_stats
    except ObjectDoesNotExist:
        original_send_page_views = True
        original_send_celery_tasks = True
        original_send_stats = True

    try:
        user_notifications_count = AnalyticsPath.objects.get(ignore_path=r"^\/user_notifications_count\/.*",)
    except ObjectDoesNotExist:
        user_notifications_count = AnalyticsPath.objects.create(ignore_path=r"^\/user_notifications_count\/.*")

    try:
        admin = AnalyticsPath.objects.get(ignore_path=r"^\/admin\/.*")
    except ObjectDoesNotExist:
        admin = AnalyticsPath.objects.create(ignore_path=r"^\/admin\/.*")

    try:
        account_activate = AnalyticsPath.objects.get(ignore_path=r"^\/account\/activate\/.*")
    except ObjectDoesNotExist:
        account_activate = AnalyticsPath.objects.create(ignore_path=r"^\/account\/activate\/.*")

    token.type = 'GA-V4'
    token.token = 'G-6LYSMYK8DE'
    token.secret = 'KLlpjLZ-SRGozS5f5wb_kw'
    token.send_page_views = original_send_page_views
    token.send_celery_tasks = original_send_celery_tasks
    token.send_stats = original_send_stats
    token.name = 'AA Team Public Google Analytics (V4)'
    token.save()
    token.ignore_paths.add(admin, user_notifications_count, account_activate)
    token.save()


def remove_aa_team_token(apps, schema_editor):
    # Have to define some code to remove this identifier
    # In case of migration rollback?
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    token = Tokens.objects.filter(token="G-6LYSMYK8DE").delete()


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0007_analyticstokens_secret'),
    ]

    operations = [migrations.RunPython(
        add_aa_team_token, remove_aa_team_token)]



"""Settings for the Discord client.

To overwrite a default set the variable in your local Django settings, e.g:

.. code:: python

    DISCORD_GUILD_NAME_CACHE_MAX_AGE = 7200
"""

from ..utils import clean_setting


DISCORD_API_BASE_URL = clean_setting(
    'DISCORD_API_BASE_URL', 'https://discord.com/api/'
)
"""Base URL for all API calls. Must end with /."""

DISCORD_API_TIMEOUT_CONNECT = clean_setting(
    'DISCORD_API_TIMEOUT', 5
)
"""Low level connect timeout for requests to the Discord API in seconds."""

DISCORD_API_TIMEOUT_READ = clean_setting(
    'DISCORD_API_TIMEOUT', 30
)
"""Low level read timeout for requests to the Discord API in seconds."""

DISCORD_OAUTH_BASE_URL = clean_setting(
    'DISCORD_OAUTH_BASE_URL', 'https://discord.com/api/oauth2/authorize'
)
"""Base authorization URL for Discord Oauth."""

DISCORD_OAUTH_TOKEN_URL = clean_setting(
    'DISCORD_OAUTH_TOKEN_URL', 'https://discord.com/api/oauth2/token'
)
"""Base authorization URL for Discord Oauth."""

DISCORD_GUILD_NAME_CACHE_MAX_AGE = clean_setting(
    'DISCORD_GUILD_NAME_CACHE_MAX_AGE', 3600 * 24
)
"""How long the Discord guild names retrieved from the server
are caches locally in seconds.
"""

DISCORD_ROLES_CACHE_MAX_AGE = clean_setting(
    'DISCORD_ROLES_CACHE_MAX_AGE', 3600 * 1
)
"""How long Discord roles retrieved from the server are caches locally in seconds."""

DISCORD_DISABLE_ROLE_CREATION = clean_setting(
    'DISCORD_DISABLE_ROLE_CREATION', False
)
"""Turns off creation of new roles. In case the rate limit for creating roles is
exhausted, this setting allows the Discord service to continue to function
and wait out the reset. Rate limit is about 250 per 48 hrs.
"""



# Generated by Django 4.0.2 on 2022-02-26 03:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0019_merge_20211026_0919'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('de', 'German'), ('es', 'Spanish'), ('zh-hans', 'Chinese Simplified'), ('ru', 'Russian'), ('ko', 'Korean'), ('fr', 'French'), ('ja', 'Japanese'), ('it', 'Italian')], default='', max_length=10, verbose_name='Language'),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='night_mode',
            field=models.BooleanField(blank=True, null=True, verbose_name='Night Mode'),
        ),
    ]



"""Integration tests

Testing all components of the service, with the exception of the Discord API.

Please note that these tests require Redis and will flush it
"""
import dataclasses
import json
import logging
from unittest.mock import Mock, patch
from uuid import uuid1

import requests_mock
from requests.exceptions import HTTPError

from django.contrib.auth.models import Group, User
from django.test import TransactionTestCase, override_settings
from django.urls import reverse
from django_webtest import WebTest

from allianceauth.authentication.models import State
from allianceauth.eveonline.models import EveCharacter
from allianceauth.groupmanagement.models import ReservedGroupName
from allianceauth.notifications.models import Notification
from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.utils.cache import get_redis_client
from allianceauth.utils.testing import NoSocketsTestCase

from .. import tasks
from ..core import create_bot_client
from ..discord_client import DiscordApiBackoff
from ..discord_client.app_settings import DISCORD_API_BASE_URL
from ..discord_client.tests.factories import (
    TEST_GUILD_ID,
    TEST_USER_ID,
    TEST_USER_NAME,
    create_discord_error_response_unknown_member,
    create_discord_guild_member_object,
    create_discord_guild_object,
    create_discord_role_object,
    create_discord_user_object,
)
from ..models import DiscordUser
from . import MODULE_PATH, TEST_MAIN_ID, TEST_MAIN_NAME, add_permissions_to_members
from .factories import create_discord_user, create_user

logger = logging.getLogger('allianceauth')

ROLE_ALPHA = create_discord_role_object(id=1, name="alpha")
ROLE_BRAVO = create_discord_role_object(id=2, name="bravo")
ROLE_CHARLIE = create_discord_role_object(id=3, name="charlie")
ROLE_CHARLIE_2 = create_discord_role_object(id=4, name="Charlie")  # Discord roles are case sensitive
ROLE_MIKE = create_discord_role_object(id=13, name="mike", managed=True)
ROLE_MEMBER = create_discord_role_object(99, 'Member')
ROLE_BLUE = create_discord_role_object(98, 'Blue')


@dataclasses.dataclass(frozen=True)
class DiscordRequest:
    """Helper for comparing requests made to the Discord API."""
    method: str
    url: str
    text: str = dataclasses.field(compare=False, default=None)

    def json(self):
        return json.loads(self.text)


user_get_current_request = DiscordRequest(
    method='GET',
    url=f'{DISCORD_API_BASE_URL}users/@me'
)
guild_infos_request = DiscordRequest(
    method='GET',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}'
)
guild_roles_request = DiscordRequest(
    method='GET',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/roles'
)
create_guild_role_request = DiscordRequest(
    method='POST',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/roles'
)
guild_member_request = DiscordRequest(
    method='GET',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/{TEST_USER_ID}'
)
add_guild_member_request = DiscordRequest(
    method='PUT',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/{TEST_USER_ID}'
)
modify_guild_member_request = DiscordRequest(
    method='PATCH',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/{TEST_USER_ID}'
)
remove_guild_member_request = DiscordRequest(
    method='DELETE',
    url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/{TEST_USER_ID}'
)


def clear_cache():
    redis = get_redis_client()
    redis.flushall()
    logger.info('Cache flushed')


def reset_testdata():
    AuthUtils.disconnect_signals()
    Group.objects.all().delete()
    User.objects.all().delete()
    State.objects.all().delete()
    EveCharacter.objects.all().delete()
    AuthUtils.connect_signals()
    Notification.objects.all().delete()


@patch(MODULE_PATH + '.core.DISCORD_GUILD_ID', TEST_GUILD_ID)
@patch(MODULE_PATH + '.models.DISCORD_GUILD_ID', TEST_GUILD_ID)
@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=False)
@requests_mock.Mocker()
class TestServiceFeatures(TransactionTestCase):
    fixtures = ['disable_analytics.json']

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.maxDiff = None

    def setUp(self):
        """All tests: Given a user with member state,
        service permission and active Discord account
        """
        clear_cache()
        reset_testdata()
        self.group_charlie = Group.objects.create(name='charlie')

        # States
        self.member_state = AuthUtils.get_member_state()
        self.guest_state = AuthUtils.get_guest_state()
        self.blue_state = AuthUtils.create_state("Blue", 50)
        permission = AuthUtils.get_permission_by_name('discord.access_discord')
        self.member_state.permissions.add(permission)
        self.blue_state.permissions.add(permission)

        # Test user
        self.user = AuthUtils.create_user(TEST_USER_NAME)
        self.main = AuthUtils.add_main_character_2(
            self.user,
            TEST_MAIN_NAME,
            TEST_MAIN_ID,
            corp_id='2',
            corp_name='test_corp',
            corp_ticker='TEST',
            disconnect_signals=True
        )
        self.member_state.member_characters.add(self.main)

        # verify user is a member and has an account
        self.user = User.objects.get(pk=self.user.pk)
        self.assertEqual(self.user.profile.state, self.member_state)

        self.discord_user = DiscordUser.objects.create(user=self.user, uid=TEST_USER_ID)
        self.assertTrue(DiscordUser.objects.user_has_account(self.user))

    @patch(MODULE_PATH + '.auth_hooks.DISCORD_SYNC_NAMES', True)
    def test_when_name_of_main_changes_then_discord_nick_is_updated(
        self, requests_mocker
    ):
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)

        # changing nick to trigger signals
        new_nick = f'Testnick {uuid1().hex}'[:32]
        self.user.profile.main_character.character_name = new_nick
        self.user.profile.main_character.save()

        # verify Discord nick was updates
        nick_updated = False
        for r in requests_mocker.request_history:
            my_request = DiscordRequest(r.method, r.url)
            if my_request == modify_guild_member_request and "nick" in r.json():
                nick_updated = True
                self.assertEqual(r.json()["nick"], new_nick)

        self.assertTrue(nick_updated)
        self.assertTrue(DiscordUser.objects.user_has_account(self.user))

    @patch(MODULE_PATH + '.auth_hooks.DISCORD_SYNC_NAMES', True)
    def test_when_name_of_main_changes_and_user_deleted_then_account_is_deleted(
        self, requests_mocker
    ):
        requests_mocker.patch(
            modify_guild_member_request.url, status_code=404, json={'code': 10007}
        )
        requests_mocker.delete(remove_guild_member_request.url, status_code=204)

        # changing nick to trigger signals
        new_nick = f'Testnick {uuid1().hex}'[:32]
        self.user.profile.main_character.character_name = new_nick
        self.user.profile.main_character.save()

        self.assertFalse(DiscordUser.objects.user_has_account(self.user))

    def test_when_name_of_main_changes_and_and_rate_limited_then_dont_call_api(
        self, requests_mocker
    ):
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)

        # exhausting rate limit
        client = create_bot_client()
        client._redis.set(
            name=client._KEY_GLOBAL_RATE_LIMIT_REMAINING,
            value=0,
            px=2000
        )

        # changing nick to trigger signals
        new_nick = f'Testnick {uuid1().hex}'[:32]
        self.user.profile.main_character.character_name = new_nick
        self.user.profile.main_character.save()

        # should not have called the API
        requests_made = [
            DiscordRequest(r.method, r.url) for r in requests_mocker.request_history
        ]
        self.assertListEqual(requests_made, list())

    def test_when_member_is_demoted_to_guest_then_his_account_is_deleted(
        self, requests_mocker
    ):
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)
        requests_mocker.delete(remove_guild_member_request.url, status_code=204)

        # our user is a member and has an account
        self.assertTrue(self.user.has_perm('discord.access_discord'))

        # now we demote him to guest
        self.member_state.member_characters.remove(self.main)

        # verify user is now guest
        self.user = User.objects.get(pk=self.user.pk)
        self.assertEqual(self.user.profile.state, AuthUtils.get_guest_state())

        # verify user has no longer access to Discord and no account
        self.assertFalse(self.user.has_perm('discord.access_discord'))
        self.assertFalse(DiscordUser.objects.user_has_account(self.user))

        # verify account was actually deleted from Discord server
        requests_made = [
            DiscordRequest(r.method, r.url) for r in requests_mocker.request_history
        ]
        self.assertIn(remove_guild_member_request, requests_made)

        # verify user has been notified
        self.assertTrue(Notification.objects.filter(user=self.user).exists())

    def test_when_member_changes_to_blue_state_then_roles_are_updated_accordingly(
        self, requests_mocker
    ):
        # request mocks
        requests_mocker.get(
            guild_member_request.url,
            json=create_discord_guild_member_object(roles=[3, 13, 99])
        )
        requests_mocker.get(
            guild_roles_request.url,
            json=[
                ROLE_ALPHA, ROLE_BRAVO, ROLE_CHARLIE, ROLE_MIKE, ROLE_MEMBER, ROLE_BLUE
            ]
        )
        requests_mocker.post(create_guild_role_request.url, json=ROLE_CHARLIE)
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)

        AuthUtils.disconnect_signals()
        self.user.groups.add(self.group_charlie)
        AuthUtils.connect_signals()

        # demote user to blue state
        self.blue_state.member_characters.add(self.main)
        self.member_state.member_characters.remove(self.main)

        # verify roles for user where updated
        roles_updated = False
        for r in requests_mocker.request_history:
            my_request = DiscordRequest(r.method, r.url)
            if my_request == modify_guild_member_request and "roles" in r.json():
                roles_updated = True
                self.assertSetEqual(set(r.json()["roles"]), {3, 13, 98})
                break

        self.assertTrue(roles_updated)
        self.assertTrue(DiscordUser.objects.user_has_account(self.user))

    def test_when_group_added_to_member_and_role_known_then_his_roles_are_updated(
        self, requests_mocker
    ):
        requests_mocker.get(
            guild_member_request.url,
            json=create_discord_guild_member_object(roles=[13, 99])
        )
        requests_mocker.get(
            guild_roles_request.url,
            json=[ROLE_ALPHA, ROLE_BRAVO, ROLE_CHARLIE, ROLE_MIKE, ROLE_MEMBER]
        )
        requests_mocker.post(create_guild_role_request.url, json=ROLE_CHARLIE)
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)

        # adding new group to trigger signals
        self.user.groups.add(self.group_charlie)

        # verify roles for user where updated
        roles_updated = False
        for r in requests_mocker.request_history:
            my_request = DiscordRequest(r.method, r.url)
            if my_request == modify_guild_member_request and "roles" in r.json():
                roles_updated = True
                self.assertSetEqual(set(r.json()["roles"]), {3, 13, 99})
                break

        self.assertTrue(roles_updated)
        self.assertTrue(DiscordUser.objects.user_has_account(self.user))

    def test_when_group_added_to_member_and_role_unknown_then_his_roles_are_updated(
        self, requests_mocker
    ):
        requests_mocker.get(
            guild_member_request.url,
            json=create_discord_guild_member_object(roles=['13', '99'])
        )
        requests_mocker.get(
            guild_roles_request.url,
            json=[ROLE_ALPHA, ROLE_BRAVO, ROLE_MIKE, ROLE_MEMBER]
        )
        requests_mocker.post(create_guild_role_request.url, json=ROLE_CHARLIE)
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)

        # adding new group to trigger signals
        self.user.groups.add(self.group_charlie)
        self.user.refresh_from_db()

        # verify roles for user where updated
        roles_updated = False
        for r in requests_mocker.request_history:
            my_request = DiscordRequest(r.method, r.url)
            if my_request == modify_guild_member_request and "roles" in r.json():
                roles_updated = True
                self.assertSetEqual(set(r.json()["roles"]), {3, 13, 99})
                break

        self.assertTrue(roles_updated)
        self.assertTrue(DiscordUser.objects.user_has_account(self.user))


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODULE_PATH + '.managers.DISCORD_GUILD_ID', TEST_GUILD_ID)
@patch(MODULE_PATH + '.models.DISCORD_GUILD_ID', TEST_GUILD_ID)
@requests_mock.Mocker()
class TestTasks(NoSocketsTestCase):
    def test_should_update_username(self, requests_mocker):
        # given
        user = create_user()
        discord_user = create_discord_user(user)
        discord_user_obj = create_discord_user_object()
        data = create_discord_guild_member_object(user=discord_user_obj)
        requests_mocker.get(guild_member_request.url, json=data)
        # when
        tasks.update_username.delay(user.pk)
        # then
        discord_user.refresh_from_db()
        self.assertEqual(discord_user.username, discord_user_obj["username"])
        self.assertEqual(
            discord_user.discriminator, discord_user_obj["discriminator"]
        )


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODULE_PATH + '.managers.DISCORD_GUILD_ID', TEST_GUILD_ID)
@patch(MODULE_PATH + '.models.DISCORD_GUILD_ID', TEST_GUILD_ID)
@requests_mock.Mocker()
class StateTestCase(NoSocketsTestCase):
    fixtures = ['disable_analytics.json']

    def setUp(self):
        clear_cache()
        reset_testdata()

        self.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        AuthUtils.add_main_character(
            self.user,
            'Perm Test Character', '99',
            corp_id='100',
            alliance_id='200',
            corp_name='Perm Test Corp',
            alliance_name='Perm Test Alliance'
        )
        self.test_character = EveCharacter.objects.get(character_id='99')
        self.member_state = State.objects.create(
            name='Test Member',
            priority=150,
        )
        self.access_discord = AuthUtils.get_permission_by_name('discord.access_discord')
        self.member_state.permissions.add(self.access_discord)
        self.member_state.member_characters.add(self.test_character)

    def _add_discord_user(self):
        self.discord_user = DiscordUser.objects.create(
            user=self.user, uid="12345678910"
        )

    def _refresh_user(self):
        self.user = User.objects.get(pk=self.user.pk)

    def test_perm_changes_to_higher_priority_state_creation(self, requests_mocker):
        mock_url = DiscordRequest(
            method='DELETE',
            url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/12345678910'
        )
        requests_mocker.delete(mock_url.url, status_code=204)
        self._add_discord_user()
        self._refresh_user()
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        self.assertIsNotNone(self.user.discord)
        higher_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(higher_state, self.user.profile.state)
        with self.assertRaises(DiscordUser.DoesNotExist):
            self.user.discord
        higher_state.member_characters.clear()
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        with self.assertRaises(DiscordUser.DoesNotExist):
            self.user.discord

    def test_perm_changes_to_lower_priority_state_creation(self, requests_mocker):
        mock_url = DiscordRequest(
            method='DELETE',
            url=f'{DISCORD_API_BASE_URL}guilds/{TEST_GUILD_ID}/members/12345678910'
        )
        requests_mocker.delete(mock_url.url, status_code=204)
        self._add_discord_user()
        self._refresh_user()
        lower_state = State.objects.create(
            name='Lower State',
            priority=125,
        )
        self.assertIsNotNone(self.user.discord)
        lower_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        self.member_state.member_characters.clear()
        self._refresh_user()
        self.assertEqual(lower_state, self.user.profile.state)
        with self.assertRaises(DiscordUser.DoesNotExist):
            self.user.discord
        self.member_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        with self.assertRaises(DiscordUser.DoesNotExist):
            self.user.discord


@patch(MODULE_PATH + '.core.DISCORD_GUILD_ID', TEST_GUILD_ID)
@patch(MODULE_PATH + '.managers.DISCORD_GUILD_ID', TEST_GUILD_ID)
@patch(MODULE_PATH + '.models.DISCORD_GUILD_ID', TEST_GUILD_ID)
@requests_mock.Mocker()
class TestUserFeatures(WebTest):
    fixtures = ['disable_analytics.json']

    def setUp(self):
        clear_cache()
        reset_testdata()
        self.member = AuthUtils.create_member(TEST_USER_NAME)
        AuthUtils.add_main_character_2(
            self.member,
            TEST_MAIN_NAME,
            TEST_MAIN_ID,
            disconnect_signals=True
        )
        add_permissions_to_members()

    @patch(MODULE_PATH + '.views.messages', spec=True)
    @patch(MODULE_PATH + '.managers.OAuth2Session', spec=True)
    def test_user_activation_normal(
        self, requests_mocker, mock_OAuth2Session, mock_messages
    ):
        # setup
        requests_mocker.get(
            guild_infos_request.url, json=create_discord_guild_object()
        )
        requests_mocker.get(
            user_get_current_request.url, json=create_discord_user_object()
        )
        requests_mocker.get(
            guild_roles_request.url, json=[ROLE_ALPHA, ROLE_BRAVO, ROLE_MEMBER]
        )
        requests_mocker.get(
            guild_member_request.url,
            status_code=404,
            json=create_discord_error_response_unknown_member()
        )
        requests_mocker.put(add_guild_member_request.url, status_code=201)

        authentication_code = 'auth_code'
        oauth_url = 'https://www.example.com/oauth'
        state = ''
        mock_OAuth2Session.return_value.authorization_url.return_value = \
            oauth_url, state

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

        # user clicks Discord service activation link on page
        response = services_page.click(href=reverse('discord:activate'))

        # check we got a redirect to Discord OAuth
        self.assertRedirects(
            response, expected_url=oauth_url, fetch_redirect_response=False
        )

        # simulate Discord callback
        response = self.app.get(
            reverse('discord:callback'), params={'code': authentication_code}
        )

        # user got a success message
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)

        requests_made = list()
        for r in requests_mocker.request_history:
            obj = DiscordRequest(r.method, r.url)
            requests_made.append(obj)
        self.assertIn(add_guild_member_request, requests_made)

    @patch(MODULE_PATH + '.views.messages', spec=True)
    @patch(MODULE_PATH + '.managers.OAuth2Session', spec=True)
    def test_should_activate_existing_user_and_keep_managed_and_reserved_roles(
        self, requests_mocker, mock_OAuth2Session, mock_messages
    ):
        # setup
        requests_mocker.get(
            guild_infos_request.url, json=create_discord_guild_object()
        )
        requests_mocker.get(
            user_get_current_request.url, json=create_discord_user_object()
        )
        requests_mocker.get(
            guild_roles_request.url, json=[
                ROLE_ALPHA, ROLE_CHARLIE, ROLE_MEMBER, ROLE_MIKE
            ]
        )
        requests_mocker.get(
            guild_member_request.url,
            json=create_discord_guild_member_object(roles=[1, 3, 13])
        )
        requests_mocker.patch(modify_guild_member_request.url, status_code=204)
        ReservedGroupName.objects.create(
            name="charlie", reason="dummy", created_by="xyz"
        )

        authentication_code = 'auth_code'
        oauth_url = 'https://www.example.com/oauth'
        state = ''
        mock_OAuth2Session.return_value.authorization_url.return_value = \
            oauth_url, state

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

        # user clicks Discord service activation link on page
        response = services_page.click(href=reverse('discord:activate'))

        # check we got a redirect to Discord OAuth
        self.assertRedirects(
            response, expected_url=oauth_url, fetch_redirect_response=False
        )

        # simulate Discord callback
        response = self.app.get(
            reverse('discord:callback'), params={'code': authentication_code}
        )

        # user got a success message
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)

        my_request = None
        for r in requests_mocker.request_history:
            obj = DiscordRequest(r.method, r.url, r.text)
            if obj == modify_guild_member_request:
                my_request = obj
                break
        else:
            self.fail("Request not found")
        self.assertSetEqual(set(my_request.json()["roles"]), {3, 13, 99})

    @patch(MODULE_PATH + '.views.messages', spec=True)
    @patch(MODULE_PATH + '.managers.OAuth2Session', spec=True)
    def test_user_activation_failed(
        self, requests_mocker, mock_OAuth2Session, mock_messages
    ):
        # setup
        requests_mocker.get(
            guild_infos_request.url, json=create_discord_guild_object()
        )
        requests_mocker.get(
            user_get_current_request.url, json=create_discord_user_object()
        )
        requests_mocker.get(
            guild_roles_request.url, json=[ROLE_ALPHA, ROLE_BRAVO, ROLE_MEMBER]
        )
        requests_mocker.get(
            guild_member_request.url,
            status_code=404,
            json=create_discord_error_response_unknown_member()
        )

        mock_exception = HTTPError('error')
        mock_exception.response = Mock()
        mock_exception.response.status_code = 503
        requests_mocker.put(add_guild_member_request.url, exc=mock_exception)

        authentication_code = 'auth_code'
        oauth_url = 'https://www.example.com/oauth'
        state = ''
        mock_OAuth2Session.return_value.authorization_url.return_value = \
            oauth_url, state

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

        # click activate on the service page
        response = services_page.click(href=reverse('discord:activate'))

        # check we got a redirect to Discord OAuth
        self.assertRedirects(
            response, expected_url=oauth_url, fetch_redirect_response=False
        )

        # simulate Discord callback
        response = self.app.get(
            reverse('discord:callback'), params={'code': authentication_code}
        )

        # user got a success message
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)

        requests_made = list()
        for r in requests_mocker.request_history:
            obj = DiscordRequest(r.method, r.url)
            requests_made.append(obj)
        self.assertIn(add_guild_member_request, requests_made)

    @patch(MODULE_PATH + '.views.messages', spec=True)
    def test_user_deactivation_normal(self, requests_mocker, mock_messages):
        # setup
        requests_mocker.get(
            guild_infos_request.url, json=create_discord_guild_object()
        )
        requests_mocker.delete(remove_guild_member_request.url, status_code=204)
        DiscordUser.objects.create(user=self.member, uid=TEST_USER_ID)

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

        # click deactivate on the service page
        response = services_page.click(href=reverse('discord:deactivate'))

        # check we got a redirect to service page
        self.assertRedirects(response, expected_url=reverse('services:services'))

        # user got a success message
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)

        requests_made = list()
        for r in requests_mocker.request_history:
            obj = DiscordRequest(r.method, r.url)
            requests_made.append(obj)
        self.assertIn(remove_guild_member_request, requests_made)

    @patch(MODULE_PATH + '.views.messages', spec=True)
    def test_user_deactivation_fails(self, requests_mocker, mock_messages):
        # setup
        requests_mocker.get(
            guild_infos_request.url, json=create_discord_guild_object()
        )
        mock_exception = HTTPError('error')
        mock_exception.response = Mock()
        mock_exception.response.status_code = 503
        requests_mocker.delete(remove_guild_member_request.url, exc=mock_exception)

        DiscordUser.objects.create(user=self.member, uid=TEST_USER_ID)

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

        # click deactivate on the service page
        response = services_page.click(href=reverse('discord:deactivate'))

        # check we got a redirect to service page
        self.assertRedirects(response, expected_url=reverse('services:services'))

        # user got a success message
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)

        requests_made = list()
        for r in requests_mocker.request_history:
            obj = DiscordRequest(r.method, r.url)
            requests_made.append(obj)
        self.assertIn(remove_guild_member_request, requests_made)

    @patch(MODULE_PATH + '.views.messages', spec=True)
    def test_user_add_new_server(self, requests_mocker, mock_messages):
        # setup
        mock_exception = HTTPError(Mock(**{"response.status_code": 400}))
        requests_mocker.get(guild_infos_request.url, exc=mock_exception)

        # login
        self.member.is_superuser = True
        self.member.is_staff = True
        self.member.save()
        self.app.set_user(self.member)

        # click deactivate on the service page
        response = self.app.get(reverse('services:services'))

        # check we got can see the page and the "link server" button
        self.assertEqual(response.status_int, 200)
        self.assertIsNotNone(response.html.find(id='btnLinkDiscordServer'))

    def test_when_server_name_fails_user_can_still_see_service_page(
        self, requests_mocker
    ):
        # setup
        requests_mocker.get(guild_infos_request.url, exc=DiscordApiBackoff(1000))

        # login
        self.app.set_user(self.member)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)

    @override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
    @patch(MODULE_PATH + ".core.default_bot_client", spec=True)
    def test_server_name_is_updated_by_task(
        self, requests_mocker, mock_bot_client
    ):
        # setup
        mock_bot_client.guild_name.return_value = "Test Guild"
        # run task to update usernames
        tasks.update_all_usernames()

        # login
        self.app.set_user(self.member)

        # disable API call to make sure server name is not retrieved from API
        mock_exception = HTTPError(Mock(**{"response.status_code": 400}))
        requests_mocker.get(guild_infos_request.url, exc=mock_exception)

        # user opens services page
        services_page = self.app.get(reverse('services:services'))
        self.assertEqual(services_page.status_code, 200)
        self.assertIn("Test Guild", services_page.text)



import logging

from django.contrib import admin

from ...admin import ServicesUserAdmin
from . import __title__
from .models import DiscordUser
from .utils import LoggerAddTag
from .auth_hooks import DiscordService

logger = LoggerAddTag(logging.getLogger(__name__), __title__)


@admin.register(DiscordUser)
class DiscordUserAdmin(ServicesUserAdmin):
    search_fields = ServicesUserAdmin.search_fields + ('uid', 'username')
    list_display = ServicesUserAdmin.list_display + ('activated', '_username', '_uid')
    list_filter = ServicesUserAdmin.list_filter + ('activated',)
    ordering = ('-activated',)

    def delete_queryset(self, request, queryset):
        for user in queryset:
            user.delete_user()

    @admin.display(description='Discord ID (UID)', ordering='uid')
    def _uid(self, obj):
        return obj.uid

    @admin.display(description='Discord Username', ordering='username')
    def _username(self, obj):
        return DiscordService.get_discord_username(
            username=obj.username, discriminator=obj.discriminator
        )



import logging
import requests
import re
from django.conf import settings
from django.core.cache import cache
from hashlib import md5
from . import providers
logger = logging.getLogger(__name__)

GROUP_CACHE_MAX_AGE = getattr(settings, 'DISCOURSE_GROUP_CACHE_MAX_AGE', 2 * 60 * 60)  # default 2 hours


class DiscourseError(Exception):
    def __init__(self, endpoint, errors):
        self.endpoint = endpoint
        self.errors = errors

    def __str__(self):
        return f"API execution failed.\nErrors: {self.errors}\nEndpoint: {self.endpoint}"


class DiscourseManager:

    def __init__(self):
        pass

    REVOKED_EMAIL = 'revoked@localhost'
    SUSPEND_DAYS = 99999
    SUSPEND_REASON = "Disabled by auth."

    @staticmethod
    def _get_groups():
        data = providers.discourse.client.groups()
        return [g for g in data if not g['automatic']]

    @staticmethod
    def _create_group(name):
        return providers.discourse.client.create_group(name=name[:20], visible=True)['basic_group']

    @staticmethod
    def _generate_cache_group_name_key(name):
        return 'DISCOURSE_GROUP_NAME__%s' % md5(name.encode('utf-8')).hexdigest()

    @staticmethod
    def _generate_cache_group_id_key(g_id):
        return 'DISCOURSE_GROUP_ID__%s' % g_id

    @staticmethod
    def __group_name_to_id(name):
        name = DiscourseManager._sanitize_groupname(name)

        def get_or_create_group():
            groups = DiscourseManager._get_groups()
            for g in groups:
                if g['name'].lower() == name.lower():
                    return g['id']
            return DiscourseManager._create_group(name)['id']

        return cache.get_or_set(DiscourseManager._generate_cache_group_name_key(name), get_or_create_group,
                                GROUP_CACHE_MAX_AGE)

    @staticmethod
    def __group_id_to_name(g_id):
        def get_group_name():
            groups = DiscourseManager._get_groups()
            for g in groups:
                if g['id'] == g_id:
                    return g['name']
            raise KeyError("Group ID %s not found on Discourse" % g_id)

        return cache.get_or_set(DiscourseManager._generate_cache_group_id_key(g_id), get_group_name,
                                GROUP_CACHE_MAX_AGE)

    @staticmethod
    def __add_user_to_group(g_id, username):
        providers.discourse.client.add_group_member(g_id, username)

    @staticmethod
    def __remove_user_from_group(g_id, uid):
        providers.discourse.client.delete_group_member(g_id, uid)

    @staticmethod
    def __generate_group_dict(names):
        group_dict = {}
        for name in names:
            group_dict[name] = DiscourseManager.__group_name_to_id(name)
        return group_dict

    @staticmethod
    def __get_user_groups(username):
        data = DiscourseManager.__get_user(username)
        return [g['id'] for g in data['groups'] if not g['automatic']]

    @staticmethod
    def __user_name_to_id(name, silent=False):
        data = DiscourseManager.__get_user(name)
        return data['user']['id']

    @staticmethod
    def __get_user(username, silent=False):
        return providers.discourse.client.user(username)

    @staticmethod
    def __activate_user(username):
        u_id = DiscourseManager.__user_name_to_id(username)
        providers.discourse.client.activate(u_id)

    @staticmethod
    def __update_user(username, **kwargs):
        u_id = DiscourseManager.__user_name_to_id(username)
        providers.discourse.client.update_user(endpoint, u_id, **kwargs)

    @staticmethod
    def __create_user(username, email, password):
        providers.discourse.client.create_user(username, username, email, password)

    @staticmethod
    def __check_if_user_exists(username):
        try:
            DiscourseManager.__user_name_to_id(username)
            return True
        except DiscourseError:
            return False

    @staticmethod
    def __suspend_user(username):
        u_id = DiscourseManager.__user_name_to_id(username)
        return providers.discourse.client.suspend(u_id, DiscourseManager.SUSPEND_DAYS, DiscourseManager.SUSPEND_REASON)

    @staticmethod
    def __unsuspend(username):
        u_id = DiscourseManager.__user_name_to_id(username)
        return providers.discourse.client.unsuspend(u_id)

    @staticmethod
    def __set_email(username, email):
        return providers.discourse.client.update_email(username, email)

    @staticmethod
    def __logout(u_id):
        return providers.discourse.client.log_out(u_id)

    @staticmethod
    def __get_user_by_external(u_id):
        data = providers.discourse.client.user_by_external_id(u_id)
        return data

    @staticmethod
    def __user_id_by_external_id(u_id):
        data = DiscourseManager.__get_user_by_external(u_id)
        return data['user']['id']

    @staticmethod
    def _sanitize_name(name):
        name = name.replace(' ', '_')
        name = name.replace("'", '')
        name = name.lstrip(' _')
        name = name[:20]
        name = name.rstrip(' _')
        return name

    @staticmethod
    def _sanitize_username(username):
        return DiscourseManager._sanitize_name(username)

    @staticmethod
    def _sanitize_groupname(name):
        name = re.sub(r'[^\w]', '', name)
        name = DiscourseManager._sanitize_name(name)
        if len(name) < 3:
            name = "Group " + name
        return name

    @staticmethod
    def update_groups(user):
        groups = [DiscourseManager._sanitize_groupname(user.profile.state.name)]
        for g in user.groups.all():
            groups.append(DiscourseManager._sanitize_groupname(str(g)))
        logger.debug(f"Updating discourse user {user} groups to {groups}")
        group_dict = DiscourseManager.__generate_group_dict(groups)
        inv_group_dict = {v: k for k, v in group_dict.items()}
        discord_user = DiscourseManager.__get_user_by_external(user.pk)
        username = discord_user['username']
        uid = discord_user['id']
        user_groups = DiscourseManager.__get_user_groups(username)
        add_groups = [group_dict[x] for x in group_dict if not group_dict[x] in user_groups]
        rem_groups = [x for x in user_groups if x not in inv_group_dict]
        if add_groups:
            logger.info(
                f"Updating discourse user {username} groups: adding {add_groups}")
            for g in add_groups:
                DiscourseManager.__add_user_to_group(g, username)
        if rem_groups:
            logger.info(
                f"Updating discourse user {username} groups: removing {rem_groups}")
            for g in rem_groups:
                DiscourseManager.__remove_user_from_group(g, uid)

    @staticmethod
    def disable_user(user):
        logger.debug("Disabling user %s Discourse access." % user)
        d_user = DiscourseManager.__get_user_by_external(user.pk)
        DiscourseManager.__logout(d_user['user']['id'])
        logger.info("Disabled user %s Discourse access." % user)
        return True



# Generated by Django 4.2.17 on 2025-01-06 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timerboard", "0006_alter_timer_objective_alter_timer_structure_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timer",
            name="structure",
            field=models.CharField(
                choices=[
                    ("POCO", "POCO"),
                    ("Orbital Skyhook", "Orbital Skyhook"),
                    ("I-HUB", "Sovereignty Hub"),
                    ("TCU", "TCU"),
                    ("POS[S]", "POS [S]"),
                    ("POS[M]", "POS [M]"),
                    ("POS[L]", "POS [L]"),
                    ("Astrahus", "Astrahus"),
                    ("Fortizar", "Fortizar"),
                    ("Keepstar", "Keepstar"),
                    ("Raitaru", "Raitaru"),
                    ("Azbel", "Azbel"),
                    ("Sotiyo", "Sotiyo"),
                    ("Athanor", "Athanor"),
                    ("Tatara", "Tatara"),
                    ("Pharolux Cyno Beacon", "Cyno Beacon"),
                    ("Tenebrex Cyno Jammer", "Cyno Jammer"),
                    ("Ansiblex Jump Gate", "Ansiblex Jump Gate"),
                    ("Mercenary Den", "Mercenary Den"),
                    ("Moon Mining Cycle", "Moon Mining Cycle"),
                    ("Metenox Moon Drill", "Metenox Moon Drill"),
                    ("Other", "Other"),
                ],
                default="Other",
                max_length=254,
            ),
        ),
    ]



from unittest import TestCase
from unittest.mock import patch

from redis import RedisError

from allianceauth.authentication.task_statistics.helpers import (
    _RedisStub, get_redis_client_or_stub,
)

MODULE_PATH = "allianceauth.authentication.task_statistics.helpers"


class TestGetRedisClient(TestCase):
    def test_should_return_mock_if_redis_not_available_1(self):
        # when
        with patch(MODULE_PATH + ".get_redis_client") as mock_get_master_client:
            mock_get_master_client.return_value.ping.side_effect = RedisError
            result = get_redis_client_or_stub()
        # then
        self.assertIsInstance(result, _RedisStub)

    def test_should_return_mock_if_redis_not_available_2(self):
        # when
        with patch(MODULE_PATH + ".get_redis_client") as mock_get_master_client:
            mock_get_master_client.return_value.ping.return_value = False
            result = get_redis_client_or_stub()
        # then
        self.assertIsInstance(result, _RedisStub)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EveonlineConfig(AppConfig):
    name = 'allianceauth.eveonline'
    label = 'eveonline'
    verbose_name = _('EVE Online')



import logging

from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import XenforoTasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class XenforoService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'xenforo'
        self.urlpatterns = urlpatterns
        self.access_perm = 'xenforo.access_xenforo'
        self.name_format = '{character_name}'

    @property
    def title(self):
        return 'XenForo Forums'

    def delete_user(self, user, notify_user=False):
        logger.debug(f'Deleting user {user} {self.name} account')
        return XenforoTasks.delete_user(user, notify_user=notify_user)

    def validate_user(self, user):
        logger.debug(f'Validating user {user} {self.name} account')
        if XenforoTasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        urls = self.Urls()
        urls.auth_activate = 'xenforo:activate'
        urls.auth_deactivate = 'xenforo:deactivate'
        urls.auth_reset_password = 'xenforo:reset_password'
        urls.auth_set_password = 'xenforo:set_password'
        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': '',
            'username': request.user.xenforo.username if XenforoTasks.has_account(request.user) else ''
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return XenforoService()



from django.contrib import admin

from .models import Application, ApplicationChoice, ApplicationComment, ApplicationForm, ApplicationQuestion, \
    ApplicationResponse


class ChoiceInline(admin.TabularInline):
    model = ApplicationChoice
    extra = 0
    verbose_name_plural = 'Choices (optional)'
    verbose_name= 'Choice'

@admin.register(ApplicationQuestion)
class QuestionAdmin(admin.ModelAdmin):
    fieldsets = [
            (None, {'fields': ['title', 'help_text', 'multi_select']}),
            ]
    inlines = [ChoiceInline]

admin.site.register(Application)
admin.site.register(ApplicationComment)
admin.site.register(ApplicationForm)
admin.site.register(ApplicationResponse)



from allianceauth.eveonline.models import EveCorporationInfo
from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils

from ..models import AutogroupsConfig
from . import patch


class AutogroupsConfigManagerTestCase(TestCase):

    def test_update_groups_for_state(self):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()
        obj.states.add(member.profile.state)

        with patch('.models.AutogroupsConfig.update_group_membership_for_user') as update_group_membership_for_user:
            AutogroupsConfig.objects.update_groups_for_state(member.profile.state)

            self.assertTrue(update_group_membership_for_user.called)
            self.assertEqual(update_group_membership_for_user.call_count, 1)
            args, kwargs = update_group_membership_for_user.call_args
            self.assertEqual(args[0], member)

    def test_update_groups_for_user(self):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()
        obj.states.add(member.profile.state)

        with patch('.models.AutogroupsConfig.update_group_membership_for_user') \
        as update_group_membership_for_user:
            AutogroupsConfig.objects.update_groups_for_user(
                user=member
            )

            self.assertTrue(update_group_membership_for_user.called)
            self.assertEqual(update_group_membership_for_user.call_count, 1)
            args, kwargs = update_group_membership_for_user.call_args
            self.assertEqual(args[0], member)

    def test_update_groups_for_user_no_state(self):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()
        obj.states.add(member.profile.state)

        with patch('.models.AutogroupsConfig.update_group_membership_for_user') \
        as update_group_membership_for_user:
            AutogroupsConfig.objects.update_groups_for_user(
                user=member,
                state=member.profile.state
            )

            self.assertTrue(update_group_membership_for_user.called)
            self.assertEqual(update_group_membership_for_user.call_count, 1)
            args, kwargs = update_group_membership_for_user.call_args
            self.assertEqual(args[0], member)


    @patch('.models.AutogroupsConfig.update_group_membership_for_user')
    @patch('.models.AutogroupsConfig.remove_user_from_alliance_groups')
    @patch('.models.AutogroupsConfig.remove_user_from_corp_groups')
    def test_update_groups_no_config(self, remove_corp, remove_alliance, update_groups):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()

        # Corp and alliance groups should be removed from users if their state has no config
        AutogroupsConfig.objects.update_groups_for_user(member)

        self.assertFalse(update_groups.called)
        self.assertTrue(remove_alliance.called)
        self.assertTrue(remove_corp.called)

        # The normal group assignment should occur if there state has a config
        obj.states.add(member.profile.state)
        AutogroupsConfig.objects.update_groups_for_user(member)

        self.assertTrue(update_groups.called)

    def test_update_group_membership_corp_in_two_configs(self):
        # given
        member = AuthUtils.create_member('test member')
        AuthUtils.add_main_character_2(
            member,
            character_id='1234',
            name='test character',
            corp_id='2345',
            corp_name='corp name',
            corp_ticker='TIKK',

        )

        corp = EveCorporationInfo.objects.create(
            corporation_id='2345',
            corporation_name='corp name',
            corporation_ticker='TIKK',
            member_count=10,
        )

        member_state = AuthUtils.get_member_state()
        member_config = AutogroupsConfig.objects.create(corp_groups=True)
        member_config.states.add(member_state)
        blue_state = AuthUtils.get_blue_state()
        blue_state.member_corporations.add(corp)
        blue_config = AutogroupsConfig.objects.create(corp_groups=True)
        blue_config.states.add(blue_state)

        member.profile.state = blue_state
        member.profile.save()

        AutogroupsConfig.objects.update_groups_for_user(member)

        # Checks before test that the role is correctly applied
        group = blue_config.get_corp_group(corp)
        self.assertIn(group, member.groups.all())

        # when
        blue_state.member_corporations.remove(corp)
        member_state.member_corporations.add(corp)
        member.profile.state = member_state
        member.profile.save()

        # then
        AutogroupsConfig.objects.update_groups_for_user(member)
        group = member_config.get_corp_group(corp)
        self.assertIn(group, member.groups.all())



from allianceauth import hooks
from allianceauth.theme.hooks import ThemeHook


CSS_STATICS = [{
    "url": "https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.3.3/css/bootstrap.min.css",
    "integrity": "sha512-jnSuA4Ss2PkkikSOLtYs8BlYIeeIK1h99ty4YfvRPAlzr377vr3CXDb7sb7eEEBYjDtcYj+AjBH3FLv5uSJuXg=="
}]

JS_STATICS = [{
    "url": "https://cdnjs.cloudflare.com/ajax/libs/popper.js/2.11.8/umd/popper.min.js",
    "integrity": "sha512-TPh2Oxlg1zp+kz3nFA0C5vVC6leG/6mm1z9+mA81MI5eaUVqasPLO8Cuk4gMF4gUfP5etR73rgU/8PNMsSesoQ=="
}, {
    "url": "https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.3.3/js/bootstrap.min.js",
    "integrity": "sha512-ykZ1QQr0Jy/4ZkvKuqWn4iF3lqPZyij9iRv6sGqLRdTPkY69YX6+7wvVGmsdBbiIfN/8OdsI7HABjvEok6ZopQ=="
}]


class BootstrapThemeHook(ThemeHook):
    """
    Bootstrap in all its glory!
    https://getbootstrap.com/
    """

    def __init__(self):
        ThemeHook.__init__(
            self,
            "Bootstrap",
            "Powerful, extensible, and feature-packed frontend toolkit.",
            html_tags={"data-theme": "bootstrap"},
            css=CSS_STATICS,
            js=JS_STATICS,
            header_padding="3.5em"
        )


class BootstrapDarkThemeHook(ThemeHook):
    """
    Bootstrap in all its glory!, but _dark_
    https://getbootstrap.com/
    """

    def __init__(self):
        ThemeHook.__init__(
            self,
            "Bootstrap Dark",
            "Powerful, extensible, and feature-packed frontend toolkit.",
            html_tags={"data-theme": "bootstrap-dark", "data-bs-theme":"dark"},
            css=CSS_STATICS,
            js=JS_STATICS,
            header_padding="3.5em"
        )


@hooks.register('theme_hook')
def register_bootstrap_dark_hook():
    return BootstrapDarkThemeHook()


@hooks.register('theme_hook')
def register_bootstrap_hook():
    return BootstrapThemeHook()



import logging

from allianceauth.services.forms import ServicePasswordModelForm
from allianceauth.services.abstract import BaseCreatePasswordServiceAccountView, BaseDeactivateServiceAccountView, \
    BaseResetPasswordServiceAccountView, BaseSetPasswordServiceAccountView
from django.conf import settings
from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Count
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render

from .models import MumbleUser

logger = logging.getLogger(__name__)


class MumblePasswordForm(ServicePasswordModelForm):
    class Meta:
        model = MumbleUser
        fields = ('password',)


class MumbleViewMixin:
    service_name = 'mumble'
    model = MumbleUser
    permission_required = 'mumble.access_mumble'


class CreateAccountMumbleView(MumbleViewMixin, BaseCreatePasswordServiceAccountView):
    pass


class DeleteMumbleView(MumbleViewMixin, BaseDeactivateServiceAccountView):
    pass


class ResetPasswordMumbleView(MumbleViewMixin, BaseResetPasswordServiceAccountView):
    pass


class SetPasswordMumbleView(MumbleViewMixin, BaseSetPasswordServiceAccountView):
    form_class = MumblePasswordForm


@login_required
@permission_required('mumble.view_connection_history')
def connection_history(request) -> HttpResponse:

    context = {
        "mumble_url": settings.MUMBLE_URL,
    }

    return render(request, 'services/mumble/mumble_connection_history.html', context)


@login_required
@permission_required("mumble.view_connection_history")
def connection_history_data(request) -> JsonResponse:
    connection_history_data = MumbleUser.objects.all(
    ).values(
        'user',
        'display_name',
        'release',
        'version',
        'last_connect',
        'last_disconnect',
    )

    return JsonResponse({"connection_history_data": list(connection_history_data)})


@login_required
@permission_required("mumble.view_connection_history")
def release_counts_data(request) -> JsonResponse:
    release_counts_data = MumbleUser.objects.values('release').annotate(user_count=Count('user_id')).order_by('release')

    return JsonResponse({
        "release_counts_data": list(release_counts_data),
    })


@login_required
@permission_required("mumble.view_connection_history")
def release_pie_chart_data(request) -> JsonResponse:
    release_counts = MumbleUser.objects.values('release').annotate(user_count=Count('user_id')).order_by('release')

    return JsonResponse({
        "labels": list(release_counts.values_list("release", flat=True)),
        "values": list(release_counts.values_list("user_count", flat=True)),
    })



"""
Forms for custom_css app
"""

# Alliance Auth Custom CSS
from allianceauth.custom_css.models import CustomCSS
from allianceauth.custom_css.widgets import CssEditorWidget

# Django
from django import forms


class CustomCSSAdminForm(forms.ModelForm):
    """
    Form for editing custom CSS
    """

    class Meta:
        model = CustomCSS
        fields = ("css",)
        widgets = {
            "css": CssEditorWidget(
                attrs={
                    "style": "width: 90%; height: 100%;",
                    "data-editor": "code-highlight",
                    "data-language": "css",
                }
            )
        }



"""
Alliance Auth User API
"""

from typing import Optional

from django.contrib.auth.models import User

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter


def get_all_characters_from_user(user: User, main_first: bool = False) -> list:
    """
    Get all characters from a user
    This function retrieves all characters associated with a given user, optionally ordering them
    with the main character first.
    If the user is None, an empty list is returned.

    :param user: The user whose characters are to be retrieved
    :type user: User
    :param main_first: If True, the main character will be listed first
    :type main_first: bool
    :return: A list of EveCharacter objects associated with the user
    :rtype: list[EveCharacter]
    """

    if user is None:
        return []

    try:
        if main_first:
            characters = [
                char.character
                for char in CharacterOwnership.objects.filter(user=user).order_by(
                    "-character__userprofile", "character__character_name"
                )
            ]
        else:
            characters = [
                char.character
                for char in CharacterOwnership.objects.filter(user=user).order_by(
                    "character__character_name"
                )
            ]
    except AttributeError:
        return []

    return characters


def get_main_character_from_user(user: User) -> Optional[EveCharacter]:
    """
    Get the main character from a user

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user is None:
        return None

    try:
        main_character = user.profile.main_character
    except AttributeError:
        return None

    return main_character


def get_main_character_name_from_user(user: User) -> str:
    """
    Get the main character name from a user

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user is None:
        sentinel_user = get_sentinel_user()

        return sentinel_user.username

    main_character = get_main_character_from_user(user=user)

    try:
        username = main_character.character_name
    except AttributeError:
        return str(user)

    return username


def get_sentinel_user() -> User:
    """
    Get the sentinel user or create one

    :return:
    """

    return User.objects.get_or_create(username="deleted")[0]



from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

import logging

logger = logging.getLogger(__name__)


class UserSettingsMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        """Django Middleware: User Settings."""

        # Intercept the built in django /setlang/ view and also save it to Database.
        # Note the annoymous user check, only logged in users will ever hit the DB here
        if request.path == '/i18n/setlang/' and not request.user.is_anonymous:
            try:
                request.user.profile.language = request.POST['language']
                request.user.profile.save()
            except Exception as e:
                logger.exception(e)

        # Only act during the login flow, _after_ user is activated (step 2: post-sso)
        elif request.path == '/sso/login' and not request.user.is_anonymous:
            # Set the Language Cookie, if it doesnt match the DB
            # Null = hasnt been set by the user ever, dont act.
            try:
                if request.user.profile.language != request.LANGUAGE_CODE and request.user.profile.language is not None:
                    response.set_cookie(key=settings.LANGUAGE_COOKIE_NAME,
                                        value=request.user.profile.language,
                                        max_age=settings.LANGUAGE_COOKIE_AGE)
            except Exception as e:
                logger.exception(e)

            # AA v3 NIGHT_MODE
            # Set our Night mode flag from the DB
            # Null = hasnt been set by the user ever, dont act.
            #
            # Night mode intercept is not needed in this middleware.
            # is saved direct to DB in NightModeRedirectView
            try:
                if request.user.profile.night_mode is not None:
                    request.session["NIGHT_MODE"] = request.user.profile.night_mode
            except Exception as e:
                logger.exception(e)

            # AA v4 Themes
            # Null = has not been set by the user ever, dont act
            # DEFAULT_THEME or DEFAULT_THEME_DARK will be used in get_theme()
            try:
                if request.user.profile.theme is not None:
                    request.session["THEME"] = request.user.profile.theme
            except Exception as e:
                logger.exception(e)

        return response



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class HRApplicationsConfig(AppConfig):
    name = 'allianceauth.hrapplications'
    label = 'hrapplications'
    verbose_name = _('HR Applications')



from django.contrib import admin
from .models import Notification


@admin.register(Notification)
class NotificationAdmin(admin.ModelAdmin):
    list_display = ("timestamp", "_main", "_state", "title", "level", "viewed")
    list_select_related = ("user", "user__profile__main_character", "user__profile__state")
    list_filter = (
        "level",
        "timestamp",
        "user__profile__state",
        ('user__profile__main_character', admin.RelatedOnlyFieldListFilter),
    )
    ordering = ("-timestamp", )
    search_fields = ["user__username", "user__profile__main_character__character_name"]

    @admin.display(
        ordering="user__profile__main_character__character_name"
    )
    def _main(self, obj):
        try:
            return obj.user.profile.main_character
        except AttributeError:
            return obj.user


    @admin.display(
        ordering="user__profile__state__name"
    )
    def _state(self, obj):
        return obj.user.profile.state


    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request) -> bool:
        return False



"""Logic for handling MenuItemHook objects."""

import hashlib
from typing import List, NamedTuple, Optional

from allianceauth.menu.hooks import MenuItemHook


class MenuItemHookCustom(MenuItemHook):
    """A user defined menu item that can be rendered with the standard template."""

    def __init__(
        self,
        text: str,
        classes: str,
        url_name: str,
        order: Optional[int] = None,
        navactive: Optional[List[str]] = None,
    ):
        super().__init__(text, classes, url_name, order, navactive)
        self.url = ""
        self.is_folder = None
        self.html_id = ""
        self.children = []


class MenuItemHookParams(NamedTuple):
    """Immutable container for params about a menu item hook."""

    text: str
    order: int
    hash: str


def generate_hash(obj: MenuItemHook) -> str:
    """Return the hash for a menu item hook."""
    my_class = obj.__class__
    name = f"{my_class.__module__}.{my_class.__name__}"
    hash_value = hashlib.sha256(name.encode("utf-8")).hexdigest()
    return hash_value


def gather_params(obj: MenuItemHook) -> MenuItemHookParams:
    """Return params from a menu item hook."""
    text = getattr(obj, "text", obj.__class__.__name__)
    order = getattr(obj, "order", None)
    hash = generate_hash(obj)
    return MenuItemHookParams(text=text, hash=hash, order=order)



# Generated by Django 2.0.2 on 2018-02-28 18:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fleetactivitytracking', '0004_make_strings_more_stringy'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fatlink',
            name='name',
        ),
        migrations.AlterField(
            model_name='fatlink',
            name='fleet',
            field=models.CharField(max_length=254),
        ),
    ]



"""Provide capability to sync menu items when needed only."""

from django.core.cache import cache

_MENU_SYNC_CACHE_KEY = "ALLIANCEAUTH-MENU-SYNCED"


def sync_menu() -> None:
    """Sync menu items if needed only."""
    from allianceauth.menu.models import MenuItem

    is_sync_needed = not _is_menu_synced() or not MenuItem.objects.exists()
    # need to also check for existence of MenuItems in database
    # to ensure the menu is synced during tests
    if is_sync_needed:
        MenuItem.objects.sync_all()
        _record_menu_was_synced()


def _is_menu_synced() -> bool:
    return cache.get(_MENU_SYNC_CACHE_KEY, False)


def _record_menu_was_synced() -> None:
    cache.set(_MENU_SYNC_CACHE_KEY, True, timeout=None)  # no timeout


def reset_menu_items_sync() -> None:
    """Ensure menu items are synced, e.g. after a Django restart."""
    cache.delete(_MENU_SYNC_CACHE_KEY)



# Generated by Django 4.2.13 on 2024-05-12 09:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0022_userprofile_theme'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('de', 'German'), ('es', 'Spanish'), ('zh-hans', 'Chinese Simplified'), ('ru', 'Russian'), ('ko', 'Korean'), ('fr', 'French'), ('ja', 'Japanese'), ('it', 'Italian'), ('uk', 'Ukrainian'), ('pl', 'Polish')], default='', max_length=10, verbose_name='Language'),
        ),
    ]



from django.urls import include, path

from . import views

app_name = 'mumble'

module_urls = [
    # Mumble service control
    path('activate/', views.CreateAccountMumbleView.as_view(), name='activate'),
    path('deactivate/', views.DeleteMumbleView.as_view(), name='deactivate'),
    path('reset_password/', views.ResetPasswordMumbleView.as_view(), name='reset_password'),
    path('set_password/', views.SetPasswordMumbleView.as_view(), name='set_password'),
    path('connection_history/', views.connection_history, name="connection_history"),
    path('ajax/connection_history_data', views.connection_history_data, name="connection_history_data"),
    path('ajax/release_counts_data', views.release_counts_data, name="release_counts_data"),
    path('ajax/release_pie_chart_data', views.release_pie_chart_data, name="release_pie_chart_data"),
]

urlpatterns = [
    path('mumble/', include((module_urls, app_name), namespace=app_name))
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class GroupManagementConfig(AppConfig):
    name = 'allianceauth.groupmanagement'
    label = 'groupmanagement'
    verbose_name = _('Group Management')

    def ready(self):
        from . import signals  # noqa: F401



from django.contrib.auth.models import User
from django.db import models
from django.utils import timezone

from allianceauth.eveonline.models import EveCharacter
from allianceauth.framework.api.user import get_sentinel_user


class Fatlink(models.Model):
    fatdatetime = models.DateTimeField(default=timezone.now)
    duration = models.PositiveIntegerField()
    fleet = models.CharField(max_length=254)
    hash = models.CharField(max_length=254, unique=True)
    creator = models.ForeignKey(User, on_delete=models.SET(get_sentinel_user))

    def __str__(self):
        return self.fleet


class Fat(models.Model):
    character = models.ForeignKey(EveCharacter, on_delete=models.CASCADE)
    fatlink = models.ForeignKey(Fatlink, on_delete=models.CASCADE)
    system = models.CharField(max_length=30)
    shiptype = models.CharField(max_length=100)
    station = models.CharField(max_length=125)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    class Meta:
        unique_together = (('character', 'fatlink'),)

    def __str__(self):
        return "Fat-link for %s" % self.character.character_name



from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.utils.testing import NoSocketsTestCase

from ..discord_client.tests.factories import TEST_USER_ID, TEST_USER_NAME
from ..models import DiscordUser
from ..utils import set_logger_to_file
from ..views import (
    activate_discord,
    deactivate_discord,
    discord_add_bot,
    discord_callback,
    reset_discord,
)
from . import MODULE_PATH, add_permissions_to_members

logger = set_logger_to_file(MODULE_PATH + '.views', __file__)


class SetupClassMixin(NoSocketsTestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.factory = RequestFactory()
        cls.user = AuthUtils.create_member(TEST_USER_NAME)
        add_permissions_to_members()
        cls.services_url = reverse('services:services')


class TestActivateDiscord(SetupClassMixin, NoSocketsTestCase):

    @patch(MODULE_PATH + '.views.DiscordUser.objects.generate_oauth_redirect_url')
    def test_redirects_to_correct_url(self, mock_generate_oauth_redirect_url):
        expected_url = '/example.com/oauth/'
        mock_generate_oauth_redirect_url.return_value = expected_url
        request = self.factory.get(reverse('discord:activate'))
        request.user = self.user
        response = activate_discord(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, expected_url)


@patch(MODULE_PATH + '.views.messages')
@patch(MODULE_PATH + '.models.create_bot_client')
class TestDeactivateDiscord(SetupClassMixin, NoSocketsTestCase):

    def setUp(self):
        DiscordUser.objects.create(user=self.user, uid=TEST_USER_ID)

    def test_when_successful_show_success_message(
        self, mock_create_bot_client, mock_messages
    ):
        # given
        mock_create_bot_client.return_value.remove_guild_member.return_value = True
        request = self.factory.get(reverse('discord:deactivate'))
        request.user = self.user
        # when
        response = deactivate_discord(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)

    def test_when_unsuccessful_show_error_message(
        self, mock_create_bot_client, mock_messages
    ):
        # given
        mock_create_bot_client.return_value.remove_guild_member.return_value = False
        request = self.factory.get(reverse('discord:deactivate'))
        request.user = self.user
        # when
        response = deactivate_discord(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)


@patch(MODULE_PATH + '.views.messages')
@patch(MODULE_PATH + '.models.create_bot_client')
class TestResetDiscord(SetupClassMixin, NoSocketsTestCase):

    def setUp(self):
        DiscordUser.objects.create(user=self.user, uid=TEST_USER_ID)

    def test_when_successful_redirect_to_activate(
        self, mock_create_bot_client, mock_messages
    ):
        # given
        mock_create_bot_client.return_value.remove_guild_member.return_value = True
        request = self.factory.get(reverse('discord:reset'))
        request.user = self.user
        # when
        response = reset_discord(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("discord:activate"))
        self.assertFalse(mock_messages.error.called)

    def test_when_unsuccessful_message_error_and_redirect_to_service(
        self, mock_create_bot_client, mock_messages
    ):
        # given
        mock_create_bot_client.return_value.remove_guild_member.return_value = False
        request = self.factory.get(reverse('discord:reset'))
        request.user = self.user
        # when
        response = reset_discord(request)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertTrue(mock_messages.error.called)


@patch(MODULE_PATH + '.views.messages')
@patch(MODULE_PATH + '.views.DiscordUser.objects.add_user')
class TestDiscordCallback(SetupClassMixin, NoSocketsTestCase):

    def setUp(self):
        DiscordUser.objects.create(user=self.user, uid=TEST_USER_ID)

    def test_success_message_when_ok(self, mock_add_user, mock_messages):
        mock_add_user.return_value = True
        request = self.factory.get(
            reverse('discord:callback'), data={'code': '1234'}
        )
        request.user = self.user
        response = discord_callback(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertTrue(mock_messages.success.called)
        self.assertFalse(mock_messages.error.called)

    def test_handle_no_code(self, mock_add_user, mock_messages):
        mock_add_user.return_value = True
        request = self.factory.get(
            reverse('discord:callback'), data={}
        )
        request.user = self.user
        response = discord_callback(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)

    def test_error_message_when_user_creation_failed(
        self, mock_add_user, mock_messages
    ):
        mock_add_user.return_value = False
        request = self.factory.get(
            reverse('discord:callback'), data={'code': '1234'}
        )
        request.user = self.user
        response = discord_callback(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.services_url)
        self.assertFalse(mock_messages.success.called)
        self.assertTrue(mock_messages.error.called)


@patch(MODULE_PATH + '.views.DiscordUser.objects.generate_bot_add_url')
class TestDiscordAddBot(NoSocketsTestCase):

    def test_add_bot(self, mock_generate_bot_add_url):
        bot_url = 'https://www.example.com/bot'
        mock_generate_bot_add_url.return_value = bot_url
        my_user = User.objects.create_superuser('Lex Luthor', 'abc', 'def')
        request = RequestFactory().get(reverse('discord:add_bot'))
        request.user = my_user
        response = discord_add_bot(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, bot_url)



from itertools import count

from django.contrib.auth.models import User

from allianceauth.menu.core import menu_item_hooks
from allianceauth.menu.models import MenuItem
from allianceauth.menu.templatetags.menu_menu_items import RenderedMenuItem
from allianceauth.services.auth_hooks import MenuItemHook
from allianceauth.tests.auth_utils import AuthUtils


def create_user(permissions=None, **kwargs) -> User:
    num = next(counter_user)
    params = {"username": f"test_user_{num}"}
    params.update(kwargs)
    user = User.objects.create(**params)
    if permissions:
        user = AuthUtils.add_permissions_to_user_by_name(perms=permissions, user=user)
    return user


def create_menu_item_hook_class(**kwargs) -> MenuItemHook:
    num = next(counter_menu_item_hook)
    return type(f"GeneratedMenuItem{num}", (MenuItemHook,), {})


def create_menu_item_hook(**kwargs) -> MenuItemHook:
    num = next(counter_menu_item_hook)
    new_class = type(f"GeneratedMenuItem{num}", (MenuItemHook,), {})

    count = kwargs.pop("count", None)
    params = {
        "text": f"Dummy App #{num}",
        "classes": "fa-solid fa-users-gear",
        "url_name": "groupmanagement:management",
    }
    params.update(kwargs)
    obj = new_class(**params)
    for key, value in params.items():
        setattr(obj, key, value)

    obj.count = count
    return obj


def create_menu_item_hook_function(**kwargs):
    obj = create_menu_item_hook(**kwargs)
    return lambda: obj


def create_link_menu_item(**kwargs) -> MenuItem:
    num = next(counter_menu_item)
    params = {
        "url": f"https://www.example.com/{num}",
    }
    params.update(kwargs)
    return _create_menu_item(**params)


def create_app_menu_item(**kwargs) -> MenuItem:
    params = {"hook_hash": "hook_hash"}
    params.update(kwargs)
    return _create_menu_item(**params)


def create_folder_menu_item(**kwargs) -> MenuItem:
    return _create_menu_item(**kwargs)


def create_menu_item_from_hook(hook, **kwargs) -> MenuItem:
    item = hook()
    hook_params = menu_item_hooks.gather_params(item)
    params = {
        "text": hook_params.text,
        "hook_hash": hook_params.hash,
        "order": hook_params.order,
    }
    params.update(kwargs)
    return _create_menu_item(**params)


def _create_menu_item(**kwargs) -> MenuItem:
    num = next(counter_menu_item)
    params = {
        "text": f"text #{num}",
    }
    params.update(kwargs)
    return MenuItem.objects.create(**params)


def create_rendered_menu_item(**kwargs) -> RenderedMenuItem:
    if "menu_item" not in kwargs:
        kwargs["menu_item"] = create_link_menu_item()

    return RenderedMenuItem(**kwargs)


counter_menu_item = count(1, 1)
counter_menu_item_hook = count(1, 1)
counter_user = count(1, 1)



from unittest import mock

from django.contrib.auth.models import User
from django.test import TestCase

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo,\
    EveAllianceInfo, EveFactionInfo
from allianceauth.tests.auth_utils import AuthUtils
from esi.errors import IncompleteResponseError
from esi.models import Token

from ..models import CharacterOwnership, State, get_guest_state
from ..tasks import check_character_ownership

MODULE_PATH = 'allianceauth.authentication'


class CharacterOwnershipTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('user', disconnect_signals=True)
        cls.alt_user = AuthUtils.create_user('alt_user', disconnect_signals=True)
        cls.character = EveCharacter.objects.create(
            character_id=1,
            character_name='Main Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )

    def test_create_ownership(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        co = CharacterOwnership.objects.get(character=self.character)
        self.assertEqual(co.user, self.user)
        self.assertEqual(co.owner_hash, '1')

    def test_transfer_ownership(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        Token.objects.create(
            user=self.alt_user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='2',
        )
        co = CharacterOwnership.objects.get(character=self.character)
        self.assertNotEqual(self.user, co.user)
        self.assertEqual(self.alt_user, co.user)

    def test_clear_main_character(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        self.user.profile.main_character = self.character
        self.user.profile.save()
        Token.objects.create(
            user=self.alt_user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='2',
        )
        self.user = User.objects.get(pk=self.user.pk)
        self.assertIsNone(self.user.profile.main_character)


class StateTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'Test Character', '1', corp_id='1', alliance_id='1',
                                    corp_name='Test Corp', alliance_name='Test Alliance', faction_id=1337,
                                    faction_name='Permabanned')
        cls.guest_state = get_guest_state()
        cls.test_character = EveCharacter.objects.get(character_id='1')
        cls.test_corporation = EveCorporationInfo.objects.create(corporation_id='1', corporation_name='Test Corp',
                                                                corporation_ticker='TEST', member_count=1)
        cls.test_alliance = EveAllianceInfo.objects.create(alliance_id='1', alliance_name='Test Alliance',
                                                            alliance_ticker='TEST', executor_corp_id='1')
        cls.test_faction = EveFactionInfo.objects.create(faction_id=1337, faction_name='Permabanned')
        cls.member_state = State.objects.create(
            name='Test Member',
            priority=150,
        )

    def _refresh_user(self):
        self.user = User.objects.get(pk=self.user.pk)

    def test_state_assignment_on_character_change(self):
        self.member_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.member_state)

        self.member_state.member_characters.remove(self.test_character)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_corporation_change(self):
        self.member_state.member_corporations.add(self.test_corporation)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.member_state)

        self.member_state.member_corporations.remove(self.test_corporation)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_alliance_addition(self):
        self.member_state.member_alliances.add(self.test_alliance)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.member_state)

        self.member_state.member_alliances.remove(self.test_alliance)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_faction_change(self):
        self.member_state.member_factions.add(self.test_faction)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.member_state)

        self.member_state.member_factions.remove(self.test_faction)
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_higher_priority_state_creation(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        higher_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(higher_state, self.user.profile.state)
        higher_state.member_characters.clear()
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        self.member_state.member_characters.clear()

    def test_state_assignment_on_lower_priority_state_creation(self):
        self.member_state.member_characters.add(self.test_character)
        lower_state = State.objects.create(
            name='Lower State',
            priority=125,
        )
        lower_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        lower_state.member_characters.clear()
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        self.member_state.member_characters.clear()

    def test_state_assignment_on_priority_change(self):
        self.member_state.member_characters.add(self.test_character)
        lower_state = State.objects.create(
            name='Lower State',
            priority=125,
        )
        lower_state.member_characters.add(self.test_character)
        self._refresh_user()
        lower_state.priority = 500
        lower_state.save()
        self._refresh_user()
        self.assertEqual(lower_state, self.user.profile.state)
        lower_state.priority = 125
        lower_state.save()
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)

    def test_state_assignment_on_state_deletion(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        higher_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEqual(higher_state, self.user.profile.state)
        higher_state.delete()
        self.assertFalse(State.objects.filter(name='Higher State').count())
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)

    def test_state_assignment_on_public_toggle(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)
        higher_state.public = True
        higher_state.save()
        self._refresh_user()
        self.assertEqual(higher_state, self.user.profile.state)
        higher_state.public = False
        higher_state.save()
        self._refresh_user()
        self.assertEqual(self.member_state, self.user.profile.state)

    def test_state_assignment_on_active_changed(self):
        self.member_state.member_characters.add(self.test_character)
        self.user.is_active = False
        self.user.save()
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.guest_state)
        self.user.is_active = True
        self.user.save()
        self._refresh_user()
        self.assertEqual(self.user.profile.state, self.member_state)


class CharacterOwnershipCheckTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'Test Character', '1', corp_id='1', alliance_id='1',
                                        corp_name='Test Corp', alliance_name='Test Alliance')
        cls.character = EveCharacter.objects.get(character_id=1)
        cls.token = Token.objects.create(
            user=cls.user,
            character_id=1,
            character_name='Test',
            character_owner_hash='1',
        )
        cls.ownership = CharacterOwnership.objects.get(character=cls.character)

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    def test_no_change_owner_hash(self, update_token_data):
        # makes sure the ownership isn't delete if owner hash hasn't changed
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    def test_unable_to_update_token_data(self, update_token_data):
        # makes sure ownerships and tokens aren't hellpurged when there's problems with the SSO servers
        update_token_data.side_effect = IncompleteResponseError()
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

        update_token_data.side_effect = KeyError()
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    @mock.patch(MODULE_PATH + '.tasks.Token.delete')
    @mock.patch(MODULE_PATH + '.tasks.Token.objects.exists')
    @mock.patch(MODULE_PATH + '.tasks.CharacterOwnership.objects.filter')
    def test_owner_hash_changed(self, filter, exists, delete, update_token_data):
        # makes sure the ownership is revoked when owner hash changes
        filter.return_value.exists.return_value = False
        check_character_ownership(self.ownership)
        self.assertTrue(filter.return_value.delete.called)



from django import forms

from .constants import DEFAULT_FOLDER_ICON_CLASSES
from .models import MenuItem


class FolderMenuItemAdminForm(forms.ModelForm):
    """A form for changing folder items."""

    class Meta:
        model = MenuItem
        fields = ["text", "classes", "order", "is_hidden"]

    def clean(self):
        data = super().clean()
        if not data["classes"]:
            data["classes"] = DEFAULT_FOLDER_ICON_CLASSES
        return data


class _BasedMenuItemAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["parent"].queryset = MenuItem.objects.filter_folders().order_by(
            "text"
        )
        self.fields["parent"].required = False
        self.fields["parent"].widget = forms.Select(
            choices=self.fields["parent"].widget.choices
        )  # disable modify buttons


class AppMenuItemAdminForm(_BasedMenuItemAdminForm):
    """A form for changing app items."""

    class Meta:
        model = MenuItem
        fields = ["order", "parent", "is_hidden"]


class LinkMenuItemAdminForm(_BasedMenuItemAdminForm):
    """A form for changing link items."""

    class Meta:
        model = MenuItem
        fields = ["text", "url", "classes", "order", "parent", "is_hidden"]
        widgets = {
            "url": forms.TextInput(attrs={"size": "100"}),
        }



# Generated by Django 4.2.16 on 2024-12-11 02:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0009_remove_analyticstokens_ignore_paths_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='analyticsidentifier',
            options={'verbose_name': 'Analytics Identifier'},
        ),
    ]



# Generated by Django 2.0.8 on 2018-12-07 08:56

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0010_authgroup_states'),
    ]

    operations = [
        migrations.AddField(
            model_name='requestlog',
            name='date',
            field=models.DateTimeField(default=datetime.datetime(2018, 12, 7, 8, 56, 33, 846342)),
        ),
    ]



from allianceauth.authentication.models import User, UserProfile
from allianceauth.eveonline.models import (
    EveCharacter,
    EveCorporationInfo,
    EveAllianceInfo
)
from django.db.models.signals import (
    pre_save,
    post_save,
    pre_delete,
    m2m_changed
)
from allianceauth.tests.auth_utils import AuthUtils

from django.test.testcases import TestCase
from unittest.mock import Mock
from . import patch


class TestUserProfileSignals(TestCase):

    def setUp(self):
        state = AuthUtils.get_member_state()

        self.char = EveCharacter.objects.create(
            character_id='1234',
            character_name='test character',
            corporation_id='2345',
            corporation_name='test corp',
            corporation_ticker='tickr',
            alliance_id='3456',
            alliance_name='alliance name',
        )

        self.alliance = EveAllianceInfo.objects.create(
            alliance_id='3456',
            alliance_name='alliance name',
            alliance_ticker='TIKR',
            executor_corp_id='2345',
        )

        self.corp = EveCorporationInfo.objects.create(
            corporation_id='2345',
            corporation_name='corp name',
            corporation_ticker='TIKK',
            member_count=10,
            alliance=self.alliance,
        )

        state.member_alliances.add(self.alliance)
        state.member_corporations.add(self.corp)

        self.member = AuthUtils.create_user('test user')
        self.member.profile.main_character = self.char
        self.member.profile.save()

    @patch('.signals.create_required_models')
    def test_create_required_models_triggered_true(
            self, create_required_models):
        """
        Create a User object here,
        to generate UserProfile models
        """
        post_save.connect(create_required_models, sender=User)
        AuthUtils.create_user('test_create_required_models_triggered')
        self.assertTrue = create_required_models.called
        self.assertEqual(create_required_models.call_count, 1)

        user = User.objects.get(username='test_create_required_models_triggered')
        self.assertIsNot(UserProfile.objects.get(user=user), False)

    @patch('.signals.create_required_models')
    def test_create_required_models_triggered_false(
            self, create_required_models):
        """
        Only call a User object Update here,
        which does not need to generate UserProfile models
        """
        post_save.connect(create_required_models, sender=User)
        char = EveCharacter.objects.create(
            character_id='1266',
            character_name='test character2',
            corporation_id='2345',
            corporation_name='test corp',
            corporation_ticker='tickr',
            alliance_id='3456',
            alliance_name='alliance name',
        )
        self.member.profile.main_character = char
        self.member.profile.save()

        self.assertTrue = create_required_models.called
        self.assertEqual(create_required_models.call_count, 0)
        self.assertIsNot(UserProfile.objects.get(user=self.member), False)
